@@ -874,6 +874,17 @@ ULONG WINAPI EventWrite( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, ULONG
     return ERROR_SUCCESS;
 }
 
+/******************************************************************************
+ * EventWriteTransfer [ADVAPI32.@]
+ */
+ULONG WINAPI EventWriteTransfer( REGHANDLE handle, PCEVENT_DESCRIPTOR descriptor, LPCGUID activity,
+                                 LPCGUID related, ULONG count, PEVENT_DATA_DESCRIPTOR data )
+{
+    FIXME("%s, %p, %s, %s, %u, %p: stub\n", wine_dbgstr_longlong(handle), descriptor,
+          debugstr_guid(activity), debugstr_guid(related), count, data);
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * EventSetInformation [ADVAPI32.@]
  */
@@ -964,12 +975,3 @@ ULONG WINAPI EnumerateTraceGuids(PTRACE_GUID_PROPERTIES *propertiesarray,
     FIXME("%p %d %p: stub\n", propertiesarray, arraycount, guidcount);
     return ERROR_INVALID_PARAMETER;
 }
-
-/******************************************************************************
- * WmiOpenBlock [ADVAPI32.@]
- */
-NTSTATUS WINAPI WmiOpenBlock(GUID *guid, ULONG access, PVOID *datablock)
-{
-    FIXME("%s %d %p: stub\n", debugstr_guid(guid), access, datablock);
-    return ERROR_SUCCESS;
-}
