@@ -20,7 +20,6 @@
 #include "wine/port.h"
 
 #include "d2d1_private.h"
-#include "wincodec.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d2d);
 
@@ -309,124 +308,25 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmap(ID2D1RenderT
     TRACE("iface %p, size {%u, %u}, src_data %p, pitch %u, desc %p, bitmap %p.\n",
             iface, size.width, size.height, src_data, pitch, desc, bitmap);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_bitmap_create(render_target->factory, render_target->device, size, src_data, pitch, desc, &object)))
+        *bitmap = &object->ID2D1Bitmap_iface;
 
-    if (FAILED(hr = d2d_bitmap_init_memory(object, render_target, size, src_data, pitch, desc)))
-    {
-        WARN("Failed to initialize bitmap, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    TRACE("Created bitmap %p.\n", object);
-    *bitmap = &object->ID2D1Bitmap_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmapFromWicBitmap(ID2D1RenderTarget *iface,
         IWICBitmapSource *bitmap_source, const D2D1_BITMAP_PROPERTIES *desc, ID2D1Bitmap **bitmap)
 {
-    const D2D1_PIXEL_FORMAT *d2d_format;
-    D2D1_BITMAP_PROPERTIES bitmap_desc;
-    WICPixelFormatGUID wic_format;
-    unsigned int bpp, data_size;
-    D2D1_SIZE_U size;
-    unsigned int i;
-    WICRect rect;
-    UINT32 pitch;
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    struct d2d_bitmap *object;
     HRESULT hr;
-    void *data;
-
-    static const struct
-    {
-        const WICPixelFormatGUID *wic;
-        D2D1_PIXEL_FORMAT d2d;
-    }
-    format_lookup[] =
-    {
-        {&GUID_WICPixelFormat32bppPBGRA, {DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_PREMULTIPLIED}},
-        {&GUID_WICPixelFormat32bppBGR,   {DXGI_FORMAT_B8G8R8A8_UNORM, D2D1_ALPHA_MODE_IGNORE}},
-    };
 
     TRACE("iface %p, bitmap_source %p, desc %p, bitmap %p.\n",
             iface, bitmap_source, desc, bitmap);
 
-    if (FAILED(hr = IWICBitmapSource_GetSize(bitmap_source, &size.width, &size.height)))
-    {
-        WARN("Failed to get bitmap size, hr %#x.\n", hr);
-        return hr;
-    }
-
-    if (!desc)
-    {
-        bitmap_desc.pixelFormat.format = DXGI_FORMAT_UNKNOWN;
-        bitmap_desc.pixelFormat.alphaMode = D2D1_ALPHA_MODE_UNKNOWN;
-        bitmap_desc.dpiX = 0.0f;
-        bitmap_desc.dpiY = 0.0f;
-    }
-    else
-    {
-        bitmap_desc = *desc;
-    }
-
-    if (FAILED(hr = IWICBitmapSource_GetPixelFormat(bitmap_source, &wic_format)))
-    {
-        WARN("Failed to get bitmap format, hr %#x.\n", hr);
-        return hr;
-    }
-
-    for (i = 0, d2d_format = NULL; i < sizeof(format_lookup) / sizeof(*format_lookup); ++i)
-    {
-        if (IsEqualGUID(&wic_format, format_lookup[i].wic))
-        {
-            d2d_format = &format_lookup[i].d2d;
-            break;
-        }
-    }
-
-    if (!d2d_format)
-    {
-        WARN("Unsupported WIC bitmap format %s.\n", debugstr_guid(&wic_format));
-        return D2DERR_UNSUPPORTED_PIXEL_FORMAT;
-    }
-
-    if (bitmap_desc.pixelFormat.format == DXGI_FORMAT_UNKNOWN)
-        bitmap_desc.pixelFormat.format = d2d_format->format;
-    if (bitmap_desc.pixelFormat.alphaMode == D2D1_ALPHA_MODE_UNKNOWN)
-        bitmap_desc.pixelFormat.alphaMode = d2d_format->alphaMode;
-
-    switch (bitmap_desc.pixelFormat.format)
-    {
-        case DXGI_FORMAT_B8G8R8A8_UNORM:
-            bpp = 4;
-            break;
-
-        default:
-            FIXME("Unhandled format %#x.\n", bitmap_desc.pixelFormat.format);
-            return D2DERR_UNSUPPORTED_PIXEL_FORMAT;
-    }
-
-    pitch = ((bpp * size.width) + 15) & ~15;
-    data_size = pitch * size.height;
-    if (!(data = HeapAlloc(GetProcessHeap(), 0, data_size)))
-        return E_OUTOFMEMORY;
-
-    rect.X = 0;
-    rect.Y = 0;
-    rect.Width = size.width;
-    rect.Height = size.height;
-    if (FAILED(hr = IWICBitmapSource_CopyPixels(bitmap_source, &rect, pitch, data_size, data)))
-    {
-        WARN("Failed to copy bitmap pixels, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, data);
-        return hr;
-    }
-
-    hr = d2d_d3d_render_target_CreateBitmap(iface, size, data, pitch, &bitmap_desc, bitmap);
-
-    HeapFree(GetProcessHeap(), 0, data);
+    if (SUCCEEDED(hr = d2d_bitmap_create_from_wic_bitmap(render_target->factory, render_target->device, bitmap_source,
+            desc, &object)))
+        *bitmap = &object->ID2D1Bitmap_iface;
 
     return hr;
 }
@@ -441,20 +341,10 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateSharedBitmap(ID2D1R
     TRACE("iface %p, iid %s, data %p, desc %p, bitmap %p.\n",
             iface, debugstr_guid(iid), data, desc, bitmap);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_bitmap_create_shared(render_target->factory, render_target->device, iid, data, desc, &object)))
+        *bitmap = &object->ID2D1Bitmap_iface;
 
-    if (FAILED(hr = d2d_bitmap_init_shared(object, render_target, iid, data, desc)))
-    {
-        WARN("Failed to initialize bitmap, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    TRACE("Created bitmap %p.\n", object);
-    *bitmap = &object->ID2D1Bitmap_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmapBrush(ID2D1RenderTarget *iface,
@@ -463,19 +353,15 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateBitmapBrush(ID2D1Re
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     struct d2d_brush *object;
+    HRESULT hr;
 
     TRACE("iface %p, bitmap %p, bitmap_brush_desc %p, brush_desc %p, brush %p.\n",
             iface, bitmap, bitmap_brush_desc, brush_desc, brush);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_bitmap_brush_create(render_target->factory, bitmap, bitmap_brush_desc, brush_desc, &object)))
+        *brush = (ID2D1BitmapBrush *)&object->ID2D1Brush_iface;
 
-    d2d_bitmap_brush_init(object, render_target->factory, bitmap, bitmap_brush_desc, brush_desc);
-
-    TRACE("Created brush %p.\n", object);
-    *brush = (ID2D1BitmapBrush *)&object->ID2D1Brush_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateSolidColorBrush(ID2D1RenderTarget *iface,
@@ -483,18 +369,14 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateSolidColorBrush(ID2
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     struct d2d_brush *object;
+    HRESULT hr;
 
     TRACE("iface %p, color %p, desc %p, brush %p.\n", iface, color, desc, brush);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_solid_color_brush_create(render_target->factory, color, desc, &object)))
+        *brush = (ID2D1SolidColorBrush *)&object->ID2D1Brush_iface;
 
-    d2d_solid_color_brush_init(object, render_target->factory, color, desc);
-
-    TRACE("Created brush %p.\n", object);
-    *brush = (ID2D1SolidColorBrush *)&object->ID2D1Brush_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateGradientStopCollection(ID2D1RenderTarget *iface,
@@ -508,20 +390,10 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateGradientStopCollect
     TRACE("iface %p, stops %p, stop_count %u, gamma %#x, extend_mode %#x, gradient %p.\n",
             iface, stops, stop_count, gamma, extend_mode, gradient);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_gradient_create(render_target->factory, stops, stop_count, gamma, extend_mode, &object)))
+        *gradient = &object->ID2D1GradientStopCollection_iface;
 
-    if (FAILED(hr = d2d_gradient_init(object, render_target->factory, stops, stop_count, gamma, extend_mode)))
-    {
-        WARN("Failed to initialize gradient, hr %#x.\n", hr);
-        HeapFree(GetProcessHeap(), 0, object);
-        return hr;
-    }
-
-    TRACE("Created gradient %p.\n", object);
-    *gradient = &object->ID2D1GradientStopCollection_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateLinearGradientBrush(ID2D1RenderTarget *iface,
@@ -530,19 +402,16 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateLinearGradientBrush
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     struct d2d_brush *object;
+    HRESULT hr;
 
     TRACE("iface %p, gradient_brush_desc %p, brush_desc %p, gradient %p, brush %p.\n",
             iface, gradient_brush_desc, brush_desc, gradient, brush);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_linear_gradient_brush_create(render_target->factory, gradient_brush_desc, brush_desc,
+        gradient, &object)))
+        *brush = (ID2D1LinearGradientBrush *)&object->ID2D1Brush_iface;
 
-    d2d_linear_gradient_brush_init(object, render_target->factory, gradient_brush_desc, brush_desc, gradient);
-
-    TRACE("Created brush %p.\n", object);
-    *brush = (ID2D1LinearGradientBrush *)&object->ID2D1Brush_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateRadialGradientBrush(ID2D1RenderTarget *iface,
@@ -577,18 +446,14 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateMesh(ID2D1RenderTar
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     struct d2d_mesh *object;
+    HRESULT hr;
 
     TRACE("iface %p, mesh %p.\n", iface, mesh);
 
-    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
-        return E_OUTOFMEMORY;
+    if (SUCCEEDED(hr = d2d_mesh_create(render_target->factory, &object)))
+        *mesh = &object->ID2D1Mesh_iface;
 
-    d2d_mesh_init(object, render_target->factory);
-
-    TRACE("Created mesh %p.\n", object);
-    *mesh = &object->ID2D1Mesh_iface;
-
-    return S_OK;
+    return hr;
 }
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawLine(ID2D1RenderTarget *iface,
@@ -2315,3 +2180,39 @@ err:
     ID2D1Factory_Release(render_target->factory);
     return hr;
 }
+
+HRESULT d2d_d3d_render_target_update_surface(ID2D1RenderTarget *iface, IDXGISurface1 *surface)
+{
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    DXGI_SURFACE_DESC surface_desc;
+    ID3D10RenderTargetView *view;
+    ID3D10Resource *resource;
+    HRESULT hr;
+
+    if (FAILED(hr = IDXGISurface1_GetDesc(surface, &surface_desc)))
+    {
+        WARN("Failed to get surface desc, hr %#x.\n", hr);
+        return hr;
+    }
+
+    if (FAILED(hr = IDXGISurface1_QueryInterface(surface, &IID_ID3D10Resource, (void **)&resource)))
+    {
+        WARN("Failed to get ID3D10Resource interface, hr %#x.\n", hr);
+        return hr;
+    }
+
+    hr = ID3D10Device_CreateRenderTargetView(render_target->device, resource, NULL, &view);
+    ID3D10Resource_Release(resource);
+    if (FAILED(hr))
+    {
+        WARN("Failed to create rendertarget view, hr %#x.\n", hr);
+        return hr;
+    }
+
+    render_target->pixel_size.width = surface_desc.Width;
+    render_target->pixel_size.height = surface_desc.Height;
+    ID3D10RenderTargetView_Release(render_target->view);
+    render_target->view = view;
+
+    return S_OK;
+}
