@@ -72,8 +72,12 @@ struct d3drm_frame
 struct d3drm_viewport
 {
     struct d3drm_object obj;
+    struct d3drm_device *device;
+    IDirect3DRMFrame *camera;
     IDirect3DRMViewport IDirect3DRMViewport_iface;
     IDirect3DRMViewport2 IDirect3DRMViewport2_iface;
+    IDirect3DViewport *d3d_viewport;
+    IDirect3DMaterial *material;
     IDirect3DRM *d3drm;
     D3DVALUE back;
     D3DVALUE front;
@@ -106,14 +110,15 @@ HRESULT d3drm_device_create_surfaces_from_clipper(struct d3drm_device *object, I
 void d3drm_device_destroy(struct d3drm_device *device) DECLSPEC_HIDDEN;
 HRESULT d3drm_device_init(struct d3drm_device *device, UINT version, IDirectDraw *ddraw,
         IDirectDrawSurface *surface, BOOL create_z_surface) DECLSPEC_HIDDEN;
-HRESULT d3drm_device_set_ddraw_device_d3d(struct d3drm_device *device,
-        IDirect3D *d3d, IDirect3DDevice *d3d_device) DECLSPEC_HIDDEN;
 
 void d3drm_object_init(struct d3drm_object *object) DECLSPEC_HIDDEN;
 HRESULT d3drm_object_add_destroy_callback(struct d3drm_object *object, D3DRMOBJECTCALLBACK cb, void *ctx) DECLSPEC_HIDDEN;
 HRESULT d3drm_object_delete_destroy_callback(struct d3drm_object *object, D3DRMOBJECTCALLBACK cb, void *ctx) DECLSPEC_HIDDEN;
 void d3drm_object_cleanup(IDirect3DRMObject *iface, struct d3drm_object *object) DECLSPEC_HIDDEN;
 
+struct d3drm_frame *unsafe_impl_from_IDirect3DRMFrame(IDirect3DRMFrame *iface) DECLSPEC_HIDDEN;
+struct d3drm_device *unsafe_impl_from_IDirect3DRMDevice3(IDirect3DRMDevice3 *iface) DECLSPEC_HIDDEN;
+
 HRESULT d3drm_texture_create(struct d3drm_texture **texture, IDirect3DRM *d3drm) DECLSPEC_HIDDEN;
 HRESULT d3drm_frame_create(struct d3drm_frame **frame, IUnknown *parent_frame, IDirect3DRM *d3drm) DECLSPEC_HIDDEN;
 HRESULT d3drm_viewport_create(struct d3drm_viewport **viewport, IDirect3DRM *d3drm) DECLSPEC_HIDDEN;
