@@ -430,7 +430,7 @@ HRESULT WINAPI DMUSIC_CreateDirectMusicImpl(LPCGUID riid, LPVOID* ret_iface, LPU
 
     DMUSIC_LockModule();
     ret = IDirectMusic8Impl_QueryInterface(&dmusic->IDirectMusic8_iface, riid, ret_iface);
-    IDirectMusic8Impl_Release(&dmusic->IDirectMusic8_iface);
+    IDirectMusic8_Release(&dmusic->IDirectMusic8_iface);
 
     return ret;
 }
