@@ -1291,16 +1291,17 @@ void opentype_get_font_properties(struct file_stream_desc *stream_desc, struct d
         USHORT version = GET_BE_WORD(tt_os2->version);
         USHORT fsSelection = GET_BE_WORD(tt_os2->fsSelection);
         USHORT usWeightClass = GET_BE_WORD(tt_os2->usWeightClass);
+        USHORT usWidthClass = GET_BE_WORD(tt_os2->usWidthClass);
 
-        if (GET_BE_WORD(tt_os2->usWidthClass) <= DWRITE_FONT_STRETCH_ULTRA_EXPANDED)
-            props->stretch = GET_BE_WORD(tt_os2->usWidthClass);
+        if (usWidthClass > DWRITE_FONT_STRETCH_UNDEFINED && usWidthClass <= DWRITE_FONT_STRETCH_ULTRA_EXPANDED)
+            props->stretch = usWidthClass;
 
         if (usWeightClass >= 1 && usWeightClass <= 9)
             usWeightClass *= 100;
 
         if (usWeightClass > DWRITE_FONT_WEIGHT_ULTRA_BLACK)
             props->weight = DWRITE_FONT_WEIGHT_ULTRA_BLACK;
-        else
+        else if (usWeightClass > 0)
             props->weight = usWeightClass;
 
         if (version >= 4 && (fsSelection & OS2_FSSELECTION_OBLIQUE))
