@@ -31,23 +31,9 @@
 WINE_DEFAULT_DEBUG_CHANNEL(hid);
 
 static const WCHAR device_enumeratorW[] = {'H','I','D',0};
+static const WCHAR separator_W[] = {'\\',0};
 static const WCHAR device_deviceid_fmtW[] = {'%','s','\\',
     'v','i','d','_','%','0','4','x','&','p','i','d','_','%', '0','4','x'};
-static const WCHAR device_instanceid_fmtW[] = {'%','s','\\',
-    'v','i','d','_','%','0','4','x','&','p','i','d','_','%',
-    '0','4','x','&','%','s','\\','%','i','&','%','s',0};
-
-typedef struct _NATIVE_DEVICE {
-    struct list entry;
-
-    DWORD vidpid;
-    DEVICE_OBJECT *PDO;
-    DEVICE_OBJECT *FDO;
-    HID_MINIDRIVER_REGISTRATION *minidriver;
-
-} NATIVE_DEVICE;
-
-static struct list tracked_devices = LIST_INIT(tracked_devices);
 
 static NTSTATUS WINAPI internalComplete(DEVICE_OBJECT *deviceObject, IRP *irp,
     void *context )
@@ -56,60 +42,37 @@ static NTSTATUS WINAPI internalComplete(DEVICE_OBJECT *deviceObject, IRP *irp,
     return STATUS_MORE_PROCESSING_REQUIRED;
 }
 
-static NTSTATUS SendDeviceIRP(DEVICE_OBJECT* device, IRP *irp)
+static NTSTATUS get_device_id(DEVICE_OBJECT *device, BUS_QUERY_ID_TYPE type, WCHAR **id)
 {
     NTSTATUS status;
     IO_STACK_LOCATION *irpsp;
-    HANDLE event = CreateEventA(NULL, FALSE, FALSE, NULL);
+    IO_STATUS_BLOCK irp_status;
+    HANDLE event;
+    IRP *irp;
 
-    irp->UserEvent = event;
+    irp = IoBuildSynchronousFsdRequest(IRP_MJ_PNP, device, NULL, 0, NULL, NULL, &irp_status);
+    if (irp == NULL)
+        return STATUS_NO_MEMORY;
+
+    irp->UserEvent = event = CreateEventA(NULL, FALSE, FALSE, NULL);
     irpsp = IoGetNextIrpStackLocation(irp);
+    irpsp->MinorFunction = IRP_MN_QUERY_ID;
+    irpsp->Parameters.QueryId.IdType = type;
     irpsp->CompletionRoutine = internalComplete;
     irpsp->Control = SL_INVOKE_ON_SUCCESS | SL_INVOKE_ON_ERROR;
 
     IoCallDriver(device, irp);
-
     if (irp->IoStatus.u.Status == STATUS_PENDING)
         WaitForSingleObject(event, INFINITE);
 
+    *id = (WCHAR*)irp->IoStatus.Information;
     status = irp->IoStatus.u.Status;
     IoCompleteRequest(irp, IO_NO_INCREMENT );
     CloseHandle(event);
+
     return status;
 }
 
-static NTSTATUS PNP_SendPnPIRP(DEVICE_OBJECT *device, UCHAR minor)
-{
-    IO_STACK_LOCATION *irpsp;
-    IO_STATUS_BLOCK irp_status;
-
-    IRP *irp = IoBuildSynchronousFsdRequest(IRP_MJ_PNP, device, NULL, 0, NULL, NULL, &irp_status);
-
-    irpsp = IoGetNextIrpStackLocation(irp);
-    irpsp->MinorFunction = minor;
-
-    irpsp->Parameters.StartDevice.AllocatedResources = NULL;
-    irpsp->Parameters.StartDevice.AllocatedResourcesTranslated = NULL;
-
-    return SendDeviceIRP(device, irp);
-}
-
-static NTSTATUS PNP_SendPowerIRP(DEVICE_OBJECT *device, DEVICE_POWER_STATE power)
-{
-    IO_STATUS_BLOCK irp_status;
-    IO_STACK_LOCATION *irpsp;
-
-    IRP *irp = IoBuildSynchronousFsdRequest(IRP_MJ_POWER, device, NULL, 0, NULL, NULL, &irp_status);
-
-    irpsp = IoGetNextIrpStackLocation(irp);
-    irpsp->MinorFunction = IRP_MN_SET_POWER;
-
-    irpsp->Parameters.Power.Type = DevicePowerState;
-    irpsp->Parameters.Power.State.DeviceState = power;
-
-    return SendDeviceIRP(device, irp);
-}
-
 NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
 {
     DEVICE_OBJECT *device = NULL;
@@ -117,26 +80,27 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     minidriver *minidriver;
     HID_DEVICE_ATTRIBUTES attr;
     BASE_DEVICE_EXTENSION *ext = NULL;
-    WCHAR serial[256];
-    WCHAR interface[256];
-    DWORD index = HID_STRING_ID_ISERIALNUMBER;
-    NATIVE_DEVICE *tracked_device, *ptr;
-    INT interface_index = 1;
     HID_DESCRIPTOR descriptor;
     BYTE *reportDescriptor;
     INT i;
+    WCHAR *PDO_id;
+    WCHAR *id_ptr;
 
-    static const WCHAR ig_fmtW[] = {'I','G','_','%','i',0};
-    static const WCHAR im_fmtW[] = {'I','M','_','%','i',0};
+    status = get_device_id(PDO, BusQueryInstanceID, &PDO_id);
+    if (status != STATUS_SUCCESS)
+    {
+        ERR("Failed to get PDO id(%x)\n",status);
+        return status;
+    }
 
-
-    TRACE("PDO add device(%p)\n", PDO);
+    TRACE("PDO add device(%p:%s)\n", PDO, debugstr_w(PDO_id));
     minidriver = find_minidriver(driver);
 
     status = HID_CreateDevice(PDO, &minidriver->minidriver, &device);
     if (status != STATUS_SUCCESS)
     {
         ERR("Failed to create HID object (%x)\n",status);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return status;
     }
 
@@ -148,14 +112,7 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     if (status != STATUS_SUCCESS)
     {
         ERR("Minidriver AddDevice failed (%x)\n",status);
-        HID_DeleteDevice(&minidriver->minidriver, device);
-        return status;
-    }
-
-    status = PNP_SendPnPIRP(device, IRP_MN_START_DEVICE);
-    if (status != STATUS_SUCCESS)
-    {
-        ERR("Minidriver IRP_MN_START_DEVICE failed (%x)\n",status);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         HID_DeleteDevice(&minidriver->minidriver, device);
         return status;
     }
@@ -166,8 +123,8 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     if (status != STATUS_SUCCESS)
     {
         ERR("Minidriver failed to get Attributes(%x)\n",status);
-        PNP_SendPnPIRP(device, IRP_MN_REMOVE_DEVICE);
         HID_DeleteDevice(&minidriver->minidriver, device);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return status;
     }
 
@@ -176,24 +133,13 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     ext->information.VersionNumber = attr.VersionNumber;
     ext->information.Polled = minidriver->minidriver.DevicesArePolled;
 
-    tracked_device = HeapAlloc(GetProcessHeap(), 0, sizeof(*tracked_device));
-    tracked_device->vidpid = MAKELONG(attr.VendorID, attr.ProductID);
-    tracked_device->PDO = PDO;
-    tracked_device->FDO = device;
-    tracked_device->minidriver = &minidriver->minidriver;
-
-    LIST_FOR_EACH_ENTRY(ptr, &tracked_devices, NATIVE_DEVICE, entry)
-        if (ptr->vidpid == tracked_device->vidpid) interface_index++;
-
-    list_add_tail(&tracked_devices, &tracked_device->entry);
-
     status = call_minidriver(IOCTL_HID_GET_DEVICE_DESCRIPTOR, device, NULL, 0,
         &descriptor, sizeof(descriptor));
     if (status != STATUS_SUCCESS)
     {
         ERR("Cannot get Device Descriptor(%x)\n",status);
-        PNP_SendPnPIRP(device, IRP_MN_REMOVE_DEVICE);
         HID_DeleteDevice(&minidriver->minidriver, device);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return status;
     }
     for (i = 0; i < descriptor.bNumDescriptors; i++)
@@ -203,8 +149,8 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     if (i >= descriptor.bNumDescriptors)
     {
         ERR("No Report Descriptor found in reply\n");
-        PNP_SendPnPIRP(device, IRP_MN_REMOVE_DEVICE);
         HID_DeleteDevice(&minidriver->minidriver, device);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return status;
     }
 
@@ -216,6 +162,7 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
         ERR("Cannot get Report Descriptor(%x)\n",status);
         HID_DeleteDevice(&minidriver->minidriver, device);
         HeapFree(GetProcessHeap(), 0, reportDescriptor);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return status;
     }
 
@@ -226,28 +173,20 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     {
         ERR("Cannot parse Report Descriptor\n");
         HID_DeleteDevice(&minidriver->minidriver, device);
+        HeapFree(GetProcessHeap(), 0, PDO_id);
         return STATUS_NOT_SUPPORTED;
     }
 
     ext->information.DescriptorSize = ext->preparseData->dwSize;
 
-    serial[0] = 0;
-    status = call_minidriver(IOCTL_HID_GET_STRING, device,
-                             &index, sizeof(DWORD), serial, sizeof(serial));
+    lstrcpyW(ext->instance_id, device_enumeratorW);
+    strcatW(ext->instance_id, separator_W);
+    /* Skip the original enumerator */
+    id_ptr = strchrW(PDO_id, '\\');
+    id_ptr++;
+    strcatW(ext->instance_id, id_ptr);
+    HeapFree(GetProcessHeap(), 0, PDO_id);
 
-    if (serial[0] == 0)
-    {
-        static const WCHAR wZeroSerial[]= {'0','0','0','0',0};
-        lstrcpyW(serial, wZeroSerial);
-    }
-
-    if (ext->preparseData->caps.UsagePage == HID_USAGE_PAGE_GENERIC &&
-        (ext->preparseData->caps.Usage == HID_USAGE_GENERIC_GAMEPAD ||
-         ext->preparseData->caps.Usage == HID_USAGE_GENERIC_JOYSTICK))
-        sprintfW(interface, ig_fmtW, interface_index);
-    else
-        sprintfW(interface, im_fmtW, interface_index);
-    sprintfW(ext->instance_id, device_instanceid_fmtW, device_enumeratorW, ext->information.VendorID, ext->information.ProductID, interface, ext->information.VersionNumber, serial);
     sprintfW(ext->device_id, device_deviceid_fmtW, device_enumeratorW, ext->information.VendorID, ext->information.ProductID);
 
     HID_LinkDevice(device);
@@ -257,33 +196,15 @@ NTSTATUS WINAPI PNP_AddDevice(DRIVER_OBJECT *driver, DEVICE_OBJECT *PDO)
     ext->ring_buffer = RingBuffer_Create(sizeof(HID_XFER_PACKET) + ext->preparseData->caps.InputReportByteLength);
 
     HID_StartDeviceThread(device);
-    PNP_SendPowerIRP(device, PowerDeviceD0);
 
     return STATUS_SUCCESS;
 }
 
-void PNP_CleanupPNP(DRIVER_OBJECT *driver)
-{
-    NATIVE_DEVICE *tracked_device, *ptr;
-
-    LIST_FOR_EACH_ENTRY_SAFE(tracked_device, ptr, &tracked_devices,
-        NATIVE_DEVICE, entry)
-    {
-        if (tracked_device->minidriver->DriverObject == driver)
-        {
-            list_remove(&tracked_device->entry);
-            PNP_SendPowerIRP(tracked_device->FDO, PowerDeviceD3);
-            PNP_SendPnPIRP(tracked_device->FDO, IRP_MN_REMOVE_DEVICE);
-            HID_DeleteDevice(tracked_device->minidriver, tracked_device->FDO);
-            HeapFree(GetProcessHeap(), 0, tracked_device);
-        }
-    }
-}
-
 NTSTATUS WINAPI HID_PNP_Dispatch(DEVICE_OBJECT *device, IRP *irp)
 {
     NTSTATUS rc = STATUS_NOT_SUPPORTED;
     IO_STACK_LOCATION *irpsp = IoGetCurrentIrpStackLocation(irp);
+    minidriver *minidriver = find_minidriver(device->DriverObject);
 
     TRACE("%p, %p\n", device, irp);
 
@@ -292,10 +213,9 @@ NTSTATUS WINAPI HID_PNP_Dispatch(DEVICE_OBJECT *device, IRP *irp)
         case IRP_MN_QUERY_ID:
         {
             BASE_DEVICE_EXTENSION *ext = device->DeviceExtension;
-            ULONG type = irpsp->Parameters.QueryId.IdType;
             WCHAR *id = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(WCHAR)*REGSTR_VAL_MAX_HCID_LEN);
-            TRACE("IRP_MN_QUERY_ID[%i]\n", type);
-            switch (type)
+            TRACE("IRP_MN_QUERY_ID[%i]\n", irpsp->Parameters.QueryId.IdType);
+            switch (irpsp->Parameters.QueryId.IdType)
             {
                 case BusQueryHardwareIDs:
                 case BusQueryCompatibleIDs:
@@ -326,13 +246,22 @@ NTSTATUS WINAPI HID_PNP_Dispatch(DEVICE_OBJECT *device, IRP *irp)
                     irp->IoStatus.Information = (ULONG_PTR)id;
                     rc = STATUS_SUCCESS;
                     break;
+                case BusQueryDeviceSerialNumber:
+                    FIXME("BusQueryDeviceSerialNumber not implemented\n");
+                    HeapFree(GetProcessHeap(), 0, id);
+                    break;
             }
             break;
         }
+        case IRP_MN_REMOVE_DEVICE:
+        {
+            HID_DeleteDevice(&minidriver->minidriver, device);
+            minidriver->PNPDispatch(device, irp);
+            break;
+        }
         default:
         {
             /* Forward IRP to the minidriver */
-            minidriver *minidriver = find_minidriver(device->DriverObject);
             return minidriver->PNPDispatch(device, irp);
         }
     }
