@@ -50,6 +50,7 @@ typedef struct {
     enum {
         EXPRVAL_JSVAL,
         EXPRVAL_IDREF,
+        EXPRVAL_STACK_REF,
         EXPRVAL_INVALID
     } type;
     union {
@@ -58,6 +59,8 @@ typedef struct {
             IDispatch *disp;
             DISPID id;
         } idref;
+        unsigned off;
+        HRESULT hres;
     } u;
 } exprval_t;
 
@@ -96,27 +99,21 @@ static inline HRESULT stack_push_string(script_ctx_t *ctx, const WCHAR *str)
     return stack_push(ctx, jsval_string(v));
 }
 
-static HRESULT stack_push_objid(script_ctx_t *ctx, IDispatch *disp, DISPID id)
-{
-    HRESULT hres;
-
-    hres = stack_push(ctx, jsval_disp(disp));
-    if(FAILED(hres))
-        return hres;
-
-    return stack_push(ctx, jsval_number(id));
-}
-
 static inline jsval_t stack_top(script_ctx_t *ctx)
 {
     assert(ctx->stack_top > ctx->call_ctx->stack_base);
     return ctx->stack[ctx->stack_top-1];
 }
 
-static inline jsval_t stack_topn(script_ctx_t *ctx, unsigned n)
+static inline jsval_t *stack_top_ref(script_ctx_t *ctx, unsigned n)
 {
     assert(ctx->stack_top > ctx->call_ctx->stack_base+n);
-    return ctx->stack[ctx->stack_top-1-n];
+    return ctx->stack+ctx->stack_top-1-n;
+}
+
+static inline jsval_t stack_topn(script_ctx_t *ctx, unsigned n)
+{
+    return *stack_top_ref(ctx, n);
 }
 
 static inline jsval_t *stack_args(script_ctx_t *ctx, unsigned n)
@@ -178,32 +175,178 @@ static inline HRESULT stack_pop_uint(script_ctx_t *ctx, DWORD *r)
     return to_uint32(ctx, stack_pop(ctx), r);
 }
 
-static inline IDispatch *stack_pop_objid(script_ctx_t *ctx, DISPID *id)
+static inline unsigned local_off(call_frame_t *frame, int ref)
 {
-    assert(is_number(stack_top(ctx)) && is_object_instance(stack_topn(ctx, 1)));
-
-    *id = get_number(stack_pop(ctx));
-    return get_object(stack_pop(ctx));
+    return ref < 0
+        ? frame->arguments_off - ref-1
+        : frame->variables_off + ref;
 }
 
-static inline IDispatch *stack_topn_objid(script_ctx_t *ctx, unsigned n, DISPID *id)
+static inline BSTR local_name(call_frame_t *frame, int ref)
 {
-    assert(is_number(stack_topn(ctx, n)) && is_object_instance(stack_topn(ctx, n+1)));
-
-    *id = get_number(stack_topn(ctx, n));
-    return get_object(stack_topn(ctx, n+1));
+    return ref < 0 ? frame->function->params[-ref-1] : frame->function->variables[ref].name;
 }
 
-static inline jsval_t steal_ret(call_frame_t *frame)
+/* Steals input reference even on failure. */
+static HRESULT stack_push_exprval(script_ctx_t *ctx, exprval_t *val)
 {
-    jsval_t r = frame->ret;
-    frame->ret = jsval_undefined();
-    return r;
+    HRESULT hres;
+
+    switch(val->type) {
+    case EXPRVAL_JSVAL:
+        assert(0);
+    case EXPRVAL_IDREF:
+        hres = stack_push(ctx, jsval_disp(val->u.idref.disp));
+        if(SUCCEEDED(hres))
+            hres = stack_push(ctx, jsval_number(val->u.idref.id));
+        else
+            IDispatch_Release(val->u.idref.disp);
+        return hres;
+    case EXPRVAL_STACK_REF:
+        hres = stack_push(ctx, jsval_number(val->u.off));
+        if(SUCCEEDED(hres))
+            hres = stack_push(ctx, jsval_undefined());
+        return hres;
+    case EXPRVAL_INVALID:
+        hres = stack_push(ctx, jsval_undefined());
+        if(SUCCEEDED(hres))
+            hres = stack_push(ctx, jsval_number(val->u.hres));
+        return hres;
+    }
+
+    assert(0);
+    return E_FAIL;
 }
 
-static inline void clear_ret(call_frame_t *frame)
+static BOOL stack_topn_exprval(script_ctx_t *ctx, unsigned n, exprval_t *r)
 {
-    jsval_release(steal_ret(frame));
+    jsval_t v = stack_topn(ctx, n+1);
+
+    switch(jsval_type(v)) {
+    case JSV_NUMBER: {
+        call_frame_t *frame = ctx->call_ctx;
+        unsigned off = get_number(v);
+
+        if(!frame->base_scope->frame && off >= frame->arguments_off) {
+            DISPID id;
+            BSTR name;
+            HRESULT hres;
+
+            /* Got stack reference in deoptimized code. Need to convert it back to variable object reference. */
+
+            assert(off < frame->variables_off + frame->function->var_cnt);
+            name = off >= frame->variables_off
+                ? frame->function->variables[off - frame->variables_off].name
+                : frame->function->params[off - frame->arguments_off];
+            hres = jsdisp_get_id(ctx->call_ctx->base_scope->jsobj, name, 0, &id);
+            if(FAILED(hres)) {
+                r->type = EXPRVAL_INVALID;
+                r->u.hres = hres;
+                return FALSE;
+            }
+
+            *stack_top_ref(ctx, n+1) = jsval_obj(jsdisp_addref(frame->base_scope->jsobj));
+            *stack_top_ref(ctx, n) = jsval_number(id);
+            r->type = EXPRVAL_IDREF;
+            r->u.idref.disp = frame->base_scope->obj;
+            r->u.idref.id = id;
+            return TRUE;
+        }
+
+        r->type = EXPRVAL_STACK_REF;
+        r->u.off = off;
+        return TRUE;
+    }
+    case JSV_OBJECT:
+        r->type = EXPRVAL_IDREF;
+        r->u.idref.disp = get_object(v);
+        assert(is_number(stack_topn(ctx, n)));
+        r->u.idref.id = get_number(stack_topn(ctx, n));
+        return TRUE;
+    case JSV_UNDEFINED:
+        r->type = EXPRVAL_INVALID;
+        assert(is_number(stack_topn(ctx, n)));
+        r->u.hres = get_number(stack_topn(ctx, n));
+        return FALSE;
+    default:
+        assert(0);
+        return FALSE;
+    }
+}
+
+static inline BOOL stack_pop_exprval(script_ctx_t *ctx, exprval_t *r)
+{
+    BOOL ret = stack_topn_exprval(ctx, 0, r);
+    ctx->stack_top -= 2;
+    return ret;
+}
+
+static HRESULT exprval_propput(script_ctx_t *ctx, exprval_t *ref, jsval_t v)
+{
+    switch(ref->type) {
+    case EXPRVAL_STACK_REF: {
+        jsval_t *r = ctx->stack + ref->u.off;
+        jsval_release(*r);
+        return jsval_copy(v, r);
+    }
+    case EXPRVAL_IDREF:
+        return disp_propput(ctx, ref->u.idref.disp, ref->u.idref.id, v);
+    default:
+        assert(0);
+        return E_FAIL;
+    }
+}
+
+static HRESULT exprval_propget(script_ctx_t *ctx, exprval_t *ref, jsval_t *r)
+{
+    switch(ref->type) {
+    case EXPRVAL_STACK_REF:
+        return jsval_copy(ctx->stack[ref->u.off], r);
+    case EXPRVAL_IDREF:
+        return disp_propget(ctx, ref->u.idref.disp, ref->u.idref.id, r);
+    default:
+        assert(0);
+        return E_FAIL;
+    }
+}
+
+static HRESULT exprval_call(script_ctx_t *ctx, exprval_t *ref, WORD flags, unsigned argc, jsval_t *argv, jsval_t *r)
+{
+    switch(ref->type) {
+    case EXPRVAL_STACK_REF: {
+        jsval_t v = ctx->stack[ref->u.off];
+
+        if(!is_object_instance(v)) {
+            FIXME("invoke %s\n", debugstr_jsval(v));
+            return E_FAIL;
+        }
+
+        return disp_call_value(ctx, get_object(v), NULL, flags, argc, argv, r);
+    }
+    case EXPRVAL_IDREF:
+        return disp_call(ctx, ref->u.idref.disp, ref->u.idref.id, flags, argc, argv, r);
+    default:
+        assert(0);
+        return E_FAIL;
+    }
+}
+
+/* ECMA-262 3rd Edition    8.7.1 */
+/* Steals input reference. */
+static HRESULT exprval_to_value(script_ctx_t *ctx, exprval_t *ref, jsval_t *r)
+{
+    HRESULT hres;
+
+    if(ref->type == EXPRVAL_JSVAL) {
+        *r = ref->u.val;
+        return S_OK;
+    }
+
+    hres = exprval_propget(ctx, ref, r);
+
+    if(ref->type == EXPRVAL_IDREF)
+        IDispatch_Release(ref->u.idref.disp);
+    return hres;
 }
 
 static void exprval_release(exprval_t *val)
@@ -216,42 +359,35 @@ static void exprval_release(exprval_t *val)
         if(val->u.idref.disp)
             IDispatch_Release(val->u.idref.disp);
         return;
+    case EXPRVAL_STACK_REF:
     case EXPRVAL_INVALID:
         return;
     }
 }
 
-/* ECMA-262 3rd Edition    8.7.1 */
-static HRESULT exprval_to_value(script_ctx_t *ctx, exprval_t *val, jsval_t *ret)
+static inline void exprval_set_exception(exprval_t *val, HRESULT hres)
 {
-    switch(val->type) {
-    case EXPRVAL_JSVAL:
-        *ret = val->u.val;
-        val->u.val = jsval_undefined();
-        return S_OK;
-    case EXPRVAL_IDREF:
-        if(!val->u.idref.disp) {
-            FIXME("throw ReferenceError\n");
-            return E_FAIL;
-        }
-
-        return disp_propget(ctx, val->u.idref.disp, val->u.idref.id, ret);
-    case EXPRVAL_INVALID:
-        assert(0);
-    }
-
-    ERR("type %d\n", val->type);
-    return E_FAIL;
+    val->type = EXPRVAL_INVALID;
+    val->u.hres = hres;
 }
 
-static void exprval_set_idref(exprval_t *val, IDispatch *disp, DISPID id)
+static inline void exprval_set_disp_ref(exprval_t *ref, IDispatch *obj, DISPID id)
 {
-    val->type = EXPRVAL_IDREF;
-    val->u.idref.disp = disp;
-    val->u.idref.id = id;
+    ref->type = EXPRVAL_IDREF;
+    IDispatch_AddRef(ref->u.idref.disp = obj);
+    ref->u.idref.id = id;
+}
 
-    if(disp)
-        IDispatch_AddRef(disp);
+static inline jsval_t steal_ret(call_frame_t *frame)
+{
+    jsval_t r = frame->ret;
+    frame->ret = jsval_undefined();
+    return r;
+}
+
+static inline void clear_ret(call_frame_t *frame)
+{
+    jsval_release(steal_ret(frame));
 }
 
 HRESULT scope_push(scope_chain_t *scope, jsdisp_t *jsobj, IDispatch *obj, scope_chain_t **ret)
@@ -267,13 +403,8 @@ HRESULT scope_push(scope_chain_t *scope, jsdisp_t *jsobj, IDispatch *obj, scope_
     IDispatch_AddRef(obj);
     new_scope->jsobj = jsobj;
     new_scope->obj = obj;
-
-    if(scope) {
-        scope_addref(scope);
-        new_scope->next = scope;
-    }else {
-        new_scope->next = NULL;
-    }
+    new_scope->frame = NULL;
+    new_scope->next = scope ? scope_addref(scope) : NULL;
 
     *ret = new_scope;
     return S_OK;
@@ -427,6 +558,41 @@ static HRESULT equal2_values(jsval_t lval, jsval_t rval, BOOL *ret)
     return S_OK;
 }
 
+/*
+ * Transfers local variables from stack to variable object.
+ * It's slow, so we want to avoid it as much as possible.
+ */
+static HRESULT detach_variable_object(script_ctx_t *ctx, call_frame_t *frame, BOOL from_release)
+{
+    unsigned i;
+    HRESULT hres;
+
+    if(!frame->base_scope || !frame->base_scope->frame)
+        return S_OK;
+
+    TRACE("detaching %p\n", frame);
+
+    assert(frame == frame->base_scope->frame);
+    assert(frame->variable_obj == frame->base_scope->jsobj);
+
+    if(!from_release && !frame->arguments_obj) {
+        hres = setup_arguments_object(ctx, frame);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    frame->base_scope->frame = NULL;
+
+    for(i = 0; i < frame->function->locals_cnt; i++) {
+        hres = jsdisp_propput_name(frame->variable_obj, frame->function->locals[i].name,
+                                   ctx->stack[local_off(frame, frame->function->locals[i].ref)]);
+        if(FAILED(hres))
+            return hres;
+    }
+
+    return S_OK;
+}
+
 static BOOL lookup_global_members(script_ctx_t *ctx, BSTR identifier, exprval_t *ret)
 {
     named_item_t *item;
@@ -438,7 +604,7 @@ static BOOL lookup_global_members(script_ctx_t *ctx, BSTR identifier, exprval_t
             hres = disp_get_id(ctx, item->disp, identifier, identifier, 0, &id);
             if(SUCCEEDED(hres)) {
                 if(ret)
-                    exprval_set_idref(ret, item->disp, id);
+                    exprval_set_disp_ref(ret, item->disp, id);
                 return TRUE;
             }
         }
@@ -447,6 +613,16 @@ static BOOL lookup_global_members(script_ctx_t *ctx, BSTR identifier, exprval_t
     return FALSE;
 }
 
+static int local_ref_cmp(const void *key, const void *ref)
+{
+    return strcmpW((const WCHAR*)key, ((const local_ref_t*)ref)->name);
+}
+
+local_ref_t *lookup_local(const function_code_t *function, const WCHAR *identifier)
+{
+    return bsearch(identifier, function->locals, function->locals_cnt, sizeof(*function->locals), local_ref_cmp);
+}
+
 /* ECMA-262 3rd Edition    10.1.4 */
 static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *ret)
 {
@@ -459,12 +635,30 @@ static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *re
 
     if(ctx->call_ctx) {
         for(scope = ctx->call_ctx->scope; scope; scope = scope->next) {
+            if(scope->frame) {
+                function_code_t *func = scope->frame->function;
+                local_ref_t *ref = lookup_local(func, identifier);
+                static const WCHAR argumentsW[] = {'a','r','g','u','m','e','n','t','s',0};
+
+                if(ref) {
+                    ret->type = EXPRVAL_STACK_REF;
+                    ret->u.off = local_off(scope->frame, ref->ref);
+                    TRACE("returning ref %d for %d\n", ret->u.off, ref->ref);
+                    return S_OK;
+                }
+
+                if(!strcmpW(identifier, argumentsW)) {
+                    hres = detach_variable_object(ctx, scope->frame, FALSE);
+                    if(FAILED(hres))
+                        return hres;
+                }
+            }
             if(scope->jsobj)
                 hres = jsdisp_get_id(scope->jsobj, identifier, fdexNameImplicit, &id);
             else
                 hres = disp_get_id(ctx, scope->obj, identifier, identifier, fdexNameImplicit, &id);
             if(SUCCEEDED(hres)) {
-                exprval_set_idref(ret, scope->obj, id);
+                exprval_set_disp_ref(ret, scope->obj, id);
                 return S_OK;
             }
         }
@@ -472,7 +666,7 @@ static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *re
 
     hres = jsdisp_get_id(ctx->global, identifier, 0, &id);
     if(SUCCEEDED(hres)) {
-        exprval_set_idref(ret, to_disp(ctx->global), id);
+        exprval_set_disp_ref(ret, to_disp(ctx->global), id);
         return S_OK;
     }
 
@@ -509,7 +703,7 @@ static HRESULT identifier_eval(script_ctx_t *ctx, BSTR identifier, exprval_t *re
     if(lookup_global_members(ctx, identifier, ret))
         return S_OK;
 
-    ret->type = EXPRVAL_INVALID;
+    exprval_set_exception(ret, JS_E_UNDEFINED_VARIABLE);
     return S_OK;
 }
 
@@ -553,28 +747,14 @@ static inline void jmp_abs(script_ctx_t *ctx, unsigned dst)
     ctx->call_ctx->ip = dst;
 }
 
-/* ECMA-262 3rd Edition    12.2 */
-static HRESULT interp_var_set(script_ctx_t *ctx)
-{
-    const BSTR name = get_op_bstr(ctx, 0);
-    jsval_t val;
-    HRESULT hres;
-
-    TRACE("%s\n", debugstr_w(name));
-
-    val = stack_pop(ctx);
-    hres = jsdisp_propput_name(ctx->call_ctx->variable_obj, name, val);
-    jsval_release(val);
-    return hres;
-}
-
 /* ECMA-262 3rd Edition    12.6.4 */
 static HRESULT interp_forin(script_ctx_t *ctx)
 {
     const HRESULT arg = get_op_uint(ctx, 0);
-    IDispatch *var_obj, *obj = NULL;
+    IDispatch *obj = NULL;
     IDispatchEx *dispex;
-    DISPID id, var_id;
+    exprval_t prop_ref;
+    DISPID id;
     BSTR name = NULL;
     HRESULT hres;
 
@@ -583,9 +763,8 @@ static HRESULT interp_forin(script_ctx_t *ctx)
     assert(is_number(stack_top(ctx)));
     id = get_number(stack_top(ctx));
 
-    var_obj = stack_topn_objid(ctx, 1, &var_id);
-    if(!var_obj) {
-        FIXME("invalid ref\n");
+    if(!stack_topn_exprval(ctx, 1, &prop_ref)) {
+        FIXME("invalid ref: %08x\n", prop_ref.u.hres);
         return E_FAIL;
     }
 
@@ -617,7 +796,7 @@ static HRESULT interp_forin(script_ctx_t *ctx)
         stack_pop(ctx);
         stack_push(ctx, jsval_number(id)); /* safe, just after pop() */
 
-        hres = disp_propput(ctx, var_obj, var_id, jsval_string(str));
+        hres = exprval_propput(ctx, &prop_ref, jsval_string(str));
         jsstr_release(str);
         if(FAILED(hres))
             return hres;
@@ -886,6 +1065,7 @@ static HRESULT interp_memberid(script_ctx_t *ctx)
     const WCHAR *name;
     jsstr_t *name_str;
     IDispatch *obj;
+    exprval_t ref;
     DISPID id;
     HRESULT hres;
 
@@ -907,35 +1087,37 @@ static HRESULT interp_memberid(script_ctx_t *ctx)
 
     hres = disp_get_id(ctx, obj, name, NULL, arg, &id);
     jsstr_release(name_str);
-    if(FAILED(hres)) {
+    if(SUCCEEDED(hres)) {
+        ref.type = EXPRVAL_IDREF;
+        ref.u.idref.disp = obj;
+        ref.u.idref.id = id;
+    }else {
         IDispatch_Release(obj);
         if(hres == DISP_E_UNKNOWNNAME && !(arg & fdexNameEnsure)) {
-            obj = NULL;
-            id = JS_E_INVALID_PROPERTY;
+            exprval_set_exception(&ref, JS_E_INVALID_PROPERTY);
+            hres = S_OK;
         }else {
             ERR("failed %08x\n", hres);
             return hres;
         }
     }
 
-    return stack_push_objid(ctx, obj, id);
+    return stack_push_exprval(ctx, &ref);
 }
 
 /* ECMA-262 3rd Edition    11.2.1 */
 static HRESULT interp_refval(script_ctx_t *ctx)
 {
-    IDispatch *disp;
+    exprval_t ref;
     jsval_t v;
-    DISPID id;
     HRESULT hres;
 
     TRACE("\n");
 
-    disp = stack_topn_objid(ctx, 0, &id);
-    if(!disp)
+    if(!stack_topn_exprval(ctx, 0, &ref))
         return throw_reference_error(ctx, JS_E_ILLEGAL_ASSIGN, NULL);
 
-    hres = disp_propget(ctx, disp, id, &v);
+    hres = exprval_propget(ctx, &ref, &v);
     if(FAILED(hres))
         return hres;
 
@@ -992,17 +1174,15 @@ static HRESULT interp_call_member(script_ctx_t *ctx)
     const unsigned argn = get_op_uint(ctx, 0);
     const int do_ret = get_op_int(ctx, 1);
     call_frame_t *frame = ctx->call_ctx;
-    IDispatch *obj;
-    DISPID id;
+    exprval_t ref;
 
     TRACE("%d %d\n", argn, do_ret);
 
-    obj = stack_topn_objid(ctx, argn, &id);
-    if(!obj)
-        return throw_type_error(ctx, id, NULL);
+    if(!stack_topn_exprval(ctx, argn, &ref))
+        return throw_type_error(ctx, ref.u.hres, NULL);
 
     clear_ret(frame);
-    return disp_call(ctx, obj, id, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
+    return exprval_call(ctx, &ref, DISPATCH_METHOD | DISPATCH_JSCRIPT_CALLEREXECSSOURCE,
             argn, stack_args(ctx, argn), do_ret ? &frame->ret : NULL);
 }
 
@@ -1017,62 +1197,109 @@ static HRESULT interp_this(script_ctx_t *ctx)
     return stack_push(ctx, jsval_disp(frame->this_obj));
 }
 
-/* ECMA-262 3rd Edition    10.1.4 */
-static HRESULT interp_ident(script_ctx_t *ctx)
+static HRESULT interp_identifier_ref(script_ctx_t *ctx, BSTR identifier, unsigned flags)
+{
+    exprval_t exprval;
+    HRESULT hres;
+
+    hres = identifier_eval(ctx, identifier, &exprval);
+    if(FAILED(hres))
+        return hres;
+
+    if(exprval.type == EXPRVAL_INVALID && (flags & fdexNameEnsure)) {
+        DISPID id;
+
+        hres = jsdisp_get_id(ctx->global, identifier, fdexNameEnsure, &id);
+        if(FAILED(hres))
+            return hres;
+
+        exprval_set_disp_ref(&exprval, to_disp(ctx->global), id);
+    }
+
+    if(exprval.type == EXPRVAL_JSVAL || exprval.type == EXPRVAL_INVALID) {
+        WARN("invalid ref\n");
+        exprval_release(&exprval);
+        exprval_set_exception(&exprval, JS_E_OBJECT_EXPECTED);
+    }
+
+    return stack_push_exprval(ctx, &exprval);
+}
+
+static HRESULT identifier_value(script_ctx_t *ctx, BSTR identifier)
 {
-    const BSTR arg = get_op_bstr(ctx, 0);
     exprval_t exprval;
     jsval_t v;
     HRESULT hres;
 
-    TRACE("%s\n", debugstr_w(arg));
-
-    hres = identifier_eval(ctx, arg, &exprval);
+    hres = identifier_eval(ctx, identifier, &exprval);
     if(FAILED(hres))
         return hres;
 
     if(exprval.type == EXPRVAL_INVALID)
-        return throw_type_error(ctx, JS_E_UNDEFINED_VARIABLE, arg);
+        return throw_type_error(ctx, exprval.u.hres, identifier);
 
     hres = exprval_to_value(ctx, &exprval, &v);
-    exprval_release(&exprval);
     if(FAILED(hres))
         return hres;
 
     return stack_push(ctx, v);
 }
 
+static HRESULT interp_local_ref(script_ctx_t *ctx)
+{
+    const int arg = get_op_int(ctx, 0);
+    const unsigned flags = get_op_uint(ctx, 1);
+    call_frame_t *frame = ctx->call_ctx;
+    exprval_t ref;
+
+    TRACE("%d\n", arg);
+
+    if(!frame->base_scope || !frame->base_scope->frame)
+        return interp_identifier_ref(ctx, local_name(frame, arg), flags);
+
+    ref.type = EXPRVAL_STACK_REF;
+    ref.u.off = local_off(frame, arg);
+    return stack_push_exprval(ctx, &ref);
+}
+
+static HRESULT interp_local(script_ctx_t *ctx)
+{
+    const int arg = get_op_int(ctx, 0);
+    call_frame_t *frame = ctx->call_ctx;
+    jsval_t copy;
+    HRESULT hres;
+
+    TRACE("%d\n", arg);
+
+    if(!frame->base_scope || !frame->base_scope->frame)
+        return identifier_value(ctx, local_name(frame, arg));
+
+    hres = jsval_copy(ctx->stack[local_off(frame, arg)], &copy);
+    if(FAILED(hres))
+        return hres;
+
+    return stack_push(ctx, copy);
+}
+
+/* ECMA-262 3rd Edition    10.1.4 */
+static HRESULT interp_ident(script_ctx_t *ctx)
+{
+    const BSTR arg = get_op_bstr(ctx, 0);
+
+    TRACE("%s\n", debugstr_w(arg));
+
+    return identifier_value(ctx, arg);
+}
+
 /* ECMA-262 3rd Edition    10.1.4 */
 static HRESULT interp_identid(script_ctx_t *ctx)
 {
     const BSTR arg = get_op_bstr(ctx, 0);
     const unsigned flags = get_op_uint(ctx, 1);
-    exprval_t exprval;
-    HRESULT hres;
 
     TRACE("%s %x\n", debugstr_w(arg), flags);
 
-    hres = identifier_eval(ctx, arg, &exprval);
-    if(FAILED(hres))
-        return hres;
-
-    if(exprval.type == EXPRVAL_INVALID && (flags & fdexNameEnsure)) {
-        DISPID id;
-
-        hres = jsdisp_get_id(ctx->global, arg, fdexNameEnsure, &id);
-        if(FAILED(hres))
-            return hres;
-
-        exprval_set_idref(&exprval, to_disp(ctx->global), id);
-    }
-
-    if(exprval.type != EXPRVAL_IDREF) {
-        WARN("invalid ref\n");
-        exprval_release(&exprval);
-        return stack_push_objid(ctx, NULL, JS_E_OBJECT_EXPECTED);
-    }
-
-    return stack_push_objid(ctx, exprval.u.idref.disp, exprval.u.idref.id);
+    return interp_identifier_ref(ctx, arg, flags);
 }
 
 /* ECMA-262 3rd Edition    7.8.1 */
@@ -1602,6 +1829,9 @@ static HRESULT interp_delete_ident(script_ctx_t *ctx)
         return hres;
 
     switch(exprval.type) {
+    case EXPRVAL_STACK_REF:
+        ret = FALSE;
+        break;
     case EXPRVAL_IDREF:
         hres = disp_delete(exprval.u.idref.disp, exprval.u.idref.id, &ret);
         IDispatch_Release(exprval.u.idref.disp);
@@ -1672,19 +1902,17 @@ static HRESULT typeof_string(jsval_t v, const WCHAR **ret)
 static HRESULT interp_typeofid(script_ctx_t *ctx)
 {
     const WCHAR *ret;
-    IDispatch *obj;
+    exprval_t ref;
     jsval_t v;
-    DISPID id;
     HRESULT hres;
 
     TRACE("\n");
 
-    obj = stack_pop_objid(ctx, &id);
-    if(!obj)
+    if(!stack_pop_exprval(ctx, &ref))
         return stack_push(ctx, jsval_string(jsstr_undefined()));
 
-    hres = disp_propget(ctx, obj, id, &v);
-    IDispatch_Release(obj);
+    hres = exprval_propget(ctx, &ref, &v);
+    exprval_release(&ref);
     if(FAILED(hres))
         return stack_push_string(ctx, unknownW);
 
@@ -1711,14 +1939,10 @@ static HRESULT interp_typeofident(script_ctx_t *ctx)
     if(FAILED(hres))
         return hres;
 
-    if(exprval.type == EXPRVAL_INVALID) {
-        hres = stack_push(ctx, jsval_string(jsstr_undefined()));
-        exprval_release(&exprval);
-        return hres;
-    }
+    if(exprval.type == EXPRVAL_INVALID)
+        return stack_push(ctx, jsval_string(jsstr_undefined()));
 
     hres = exprval_to_value(ctx, &exprval, &v);
-    exprval_release(&exprval);
     if(FAILED(hres))
         return hres;
 
@@ -1785,28 +2009,26 @@ static HRESULT interp_tonum(script_ctx_t *ctx)
 static HRESULT interp_postinc(script_ctx_t *ctx)
 {
     const int arg = get_op_int(ctx, 0);
-    IDispatch *obj;
-    DISPID id;
+    exprval_t ref;
     jsval_t v;
     HRESULT hres;
 
     TRACE("%d\n", arg);
 
-    obj = stack_pop_objid(ctx, &id);
-    if(!obj)
+    if(!stack_pop_exprval(ctx, &ref))
         return throw_type_error(ctx, JS_E_OBJECT_EXPECTED, NULL);
 
-    hres = disp_propget(ctx, obj, id, &v);
+    hres = exprval_propget(ctx, &ref, &v);
     if(SUCCEEDED(hres)) {
         double n;
 
         hres = to_number(ctx, v, &n);
         if(SUCCEEDED(hres))
-            hres = disp_propput(ctx, obj, id, jsval_number(n+(double)arg));
+            hres = exprval_propput(ctx, &ref, jsval_number(n+(double)arg));
         if(FAILED(hres))
             jsval_release(v);
     }
-    IDispatch_Release(obj);
+    exprval_release(&ref);
     if(FAILED(hres))
         return hres;
 
@@ -1817,19 +2039,17 @@ static HRESULT interp_postinc(script_ctx_t *ctx)
 static HRESULT interp_preinc(script_ctx_t *ctx)
 {
     const int arg = get_op_int(ctx, 0);
-    IDispatch *obj;
+    exprval_t ref;
     double ret;
-    DISPID id;
     jsval_t v;
     HRESULT hres;
 
     TRACE("%d\n", arg);
 
-    obj = stack_pop_objid(ctx, &id);
-    if(!obj)
+    if(!stack_pop_exprval(ctx, &ref))
         return throw_type_error(ctx, JS_E_OBJECT_EXPECTED, NULL);
 
-    hres = disp_propget(ctx, obj, id, &v);
+    hres = exprval_propget(ctx, &ref, &v);
     if(SUCCEEDED(hres)) {
         double n;
 
@@ -1837,10 +2057,10 @@ static HRESULT interp_preinc(script_ctx_t *ctx)
         jsval_release(v);
         if(SUCCEEDED(hres)) {
             ret = n+(double)arg;
-            hres = disp_propput(ctx, obj, id, jsval_number(ret));
+            hres = exprval_propput(ctx, &ref, jsval_number(ret));
         }
     }
-    IDispatch_Release(obj);
+    exprval_release(&ref);
     if(FAILED(hres))
         return hres;
 
@@ -2223,8 +2443,7 @@ static HRESULT interp_rshift2(script_ctx_t *ctx)
 /* ECMA-262 3rd Edition    11.13.1 */
 static HRESULT interp_assign(script_ctx_t *ctx)
 {
-    IDispatch *disp;
-    DISPID id;
+    exprval_t ref;
     jsval_t v;
     HRESULT hres;
 
@@ -2232,14 +2451,13 @@ static HRESULT interp_assign(script_ctx_t *ctx)
 
     v = stack_pop(ctx);
 
-    disp = stack_pop_objid(ctx, &id);
-    if(!disp) {
+    if(!stack_pop_exprval(ctx, &ref)) {
         jsval_release(v);
         return throw_reference_error(ctx, JS_E_ILLEGAL_ASSIGN, NULL);
     }
 
-    hres = disp_propput(ctx, disp, id, v);
-    IDispatch_Release(disp);
+    hres = exprval_propput(ctx, &ref, v);
+    exprval_release(&ref);
     if(FAILED(hres)) {
         jsval_release(v);
         return hres;
@@ -2252,18 +2470,16 @@ static HRESULT interp_assign(script_ctx_t *ctx)
 static HRESULT interp_assign_call(script_ctx_t *ctx)
 {
     const unsigned argc = get_op_uint(ctx, 0);
-    IDispatch *disp;
+    exprval_t ref;
     jsval_t v;
-    DISPID id;
     HRESULT hres;
 
     TRACE("%u\n", argc);
 
-    disp = stack_topn_objid(ctx, argc+1, &id);
-    if(!disp)
+    if(!stack_topn_exprval(ctx, argc+1, &ref))
         return throw_reference_error(ctx, JS_E_ILLEGAL_ASSIGN, NULL);
 
-    hres = disp_call(ctx, disp, id, DISPATCH_PROPERTYPUT, argc+1, stack_args(ctx, argc+1), NULL);
+    hres = exprval_call(ctx, &ref, DISPATCH_PROPERTYPUT, argc+1, stack_args(ctx, argc+1), NULL);
     if(FAILED(hres))
         return hres;
 
@@ -2379,23 +2595,38 @@ OP_LIST
 #undef X
 };
 
-static void release_call_frame(call_frame_t *frame)
+static void pop_call_frame(script_ctx_t *ctx)
 {
-    if(frame->arguments_obj) {
-        /* Reset arguments value to cut the reference cycle. Note that since all activation contexts have
-         * their own arguments property, it's impossible to use prototype's one during name lookup */
-        static const WCHAR argumentsW[] = {'a','r','g','u','m','e','n','t','s',0};
-        jsdisp_propput_name(frame->variable_obj, argumentsW, jsval_undefined());
-        jsdisp_release(frame->arguments_obj);
+    call_frame_t *frame = ctx->call_ctx;
+
+    frame->stack_base -= frame->pop_locals + frame->pop_variables;
+
+    assert(frame->scope == frame->base_scope);
+
+    /* If current scope will be kept alive, we need to transfer local variables to its variable object. */
+    if(frame->scope && frame->scope->ref > 1) {
+        HRESULT hres = detach_variable_object(ctx, frame, TRUE);
+        if(FAILED(hres))
+            ERR("Failed to detach variable object: %08x\n", hres);
     }
+
+    if(frame->arguments_obj)
+        detach_arguments_object(frame->arguments_obj);
+    if(frame->scope)
+        scope_release(frame->scope);
+
+    if(frame->pop_variables)
+        stack_popn(ctx, frame->pop_variables);
+    stack_popn(ctx, frame->pop_locals);
+
+    ctx->call_ctx = frame->prev_frame;
+
     if(frame->function_instance)
         jsdisp_release(frame->function_instance);
     if(frame->variable_obj)
         jsdisp_release(frame->variable_obj);
     if(frame->this_obj)
         IDispatch_Release(frame->this_obj);
-    if(frame->scope)
-        scope_release(frame->scope);
     jsval_release(frame->ret);
     release_bytecode(frame->bytecode);
     heap_free(frame);
@@ -2417,9 +2648,8 @@ static HRESULT unwind_exception(script_ctx_t *ctx, HRESULT exception_hres)
 
         stack_popn(ctx, ctx->stack_top-frame->stack_base);
 
-        ctx->call_ctx = frame->prev_frame;
         flags = frame->flags;
-        release_call_frame(frame);
+        pop_call_frame(ctx);
         if(!(flags & EXEC_RETURN_TO_INTERP))
             return exception_hres;
     }
@@ -2492,14 +2722,13 @@ static HRESULT enter_bytecode(script_ctx_t *ctx, jsval_t *r)
             assert(ctx->stack_top == frame->stack_base);
             assert(frame->scope == frame->base_scope);
 
-            ctx->call_ctx = frame->prev_frame;
             if(return_to_interp) {
-                clear_ret(ctx->call_ctx);
-                ctx->call_ctx->ret = steal_ret(frame);
+                clear_ret(frame->prev_frame);
+                frame->prev_frame->ret = steal_ret(frame);
             }else if(r) {
                 *r = steal_ret(frame);
             }
-            release_call_frame(frame);
+            pop_call_frame(ctx);
             if(!return_to_interp)
                 break;
         }else {
@@ -2523,7 +2752,6 @@ static HRESULT bind_event_target(script_ctx_t *ctx, function_code_t *func, jsdis
         return hres;
 
     hres = exprval_to_value(ctx, &exprval, &v);
-    exprval_release(&exprval);
     if(FAILED(hres))
         return hres;
 
@@ -2547,8 +2775,85 @@ static HRESULT bind_event_target(script_ctx_t *ctx, function_code_t *func, jsdis
     return hres;
 }
 
+static HRESULT setup_scope(script_ctx_t *ctx, call_frame_t *frame, scope_chain_t *scope_chain, jsdisp_t *variable_object, unsigned argc, jsval_t *argv)
+{
+    const unsigned orig_stack = ctx->stack_top;
+    scope_chain_t *scope;
+    unsigned i;
+    jsval_t v;
+    HRESULT hres;
+
+    /* If arguments are already on the stack, we may use them. */
+    if(argv + argc == ctx->stack + ctx->stack_top) {
+        frame->arguments_off = argv - ctx->stack;
+        i = argc;
+    }else {
+        frame->arguments_off = ctx->stack_top;
+        for(i = 0; i < argc; i++) {
+            hres = jsval_copy(argv[i], &v);
+            if(SUCCEEDED(hres))
+                hres = stack_push(ctx, v);
+            if(FAILED(hres)) {
+                stack_popn(ctx, i);
+                return hres;
+            }
+        }
+    }
+
+    /* If fewer than declared arguments were passed, fill remaining with undefined value. */
+    for(; i < frame->function->param_cnt; i++) {
+        hres = stack_push(ctx, jsval_undefined());
+        if(FAILED(hres)) {
+            stack_popn(ctx, ctx->stack_top - orig_stack);
+            return hres;
+        }
+    }
+
+    frame->pop_locals = ctx->stack_top - orig_stack;
+
+    frame->variables_off = ctx->stack_top;
+
+    for(i = 0; i < frame->function->var_cnt; i++) {
+        hres = stack_push(ctx, jsval_undefined());
+        if(FAILED(hres)) {
+            stack_popn(ctx, ctx->stack_top - orig_stack);
+            return hres;
+        }
+    }
+
+    frame->pop_variables = i;
+
+    hres = scope_push(scope_chain, variable_object, to_disp(variable_object), &scope);
+    if(FAILED(hres)) {
+        stack_popn(ctx, ctx->stack_top - orig_stack);
+        return hres;
+    }
+
+    for(i = 0; i < frame->function->func_cnt; i++) {
+        if(frame->function->funcs[i].name && !frame->function->funcs[i].event_target) {
+            jsdisp_t *func_obj;
+            unsigned off;
+
+            hres = create_source_function(ctx, frame->bytecode, frame->function->funcs+i, scope, &func_obj);
+            if(FAILED(hres)) {
+                stack_popn(ctx, ctx->stack_top - orig_stack);
+                scope_release(scope);
+                return hres;
+            }
+
+            off = local_off(frame, frame->function->funcs[i].local_ref);
+            jsval_release(ctx->stack[off]);
+            ctx->stack[off] = jsval_obj(func_obj);
+        }
+    }
+
+    scope->frame = frame;
+    frame->base_scope = frame->scope = scope;
+    return S_OK;
+}
+
 HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, function_code_t *function, scope_chain_t *scope,
-        IDispatch *this_obj, jsdisp_t *function_instance, jsdisp_t *variable_obj, jsdisp_t *arguments_obj, jsval_t *r)
+        IDispatch *this_obj, jsdisp_t *function_instance, jsdisp_t *variable_obj, unsigned argc, jsval_t *argv, jsval_t *r)
 {
     call_frame_t *frame;
     unsigned i;
@@ -2557,29 +2862,38 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
     for(i = 0; i < function->func_cnt; i++) {
         jsdisp_t *func_obj;
 
-        if(!function->funcs[i].name)
+        if(!function->funcs[i].event_target)
             continue;
 
         hres = create_source_function(ctx, bytecode, function->funcs+i, scope, &func_obj);
         if(FAILED(hres))
             return hres;
 
-        if(function->funcs[i].event_target)
-            hres = bind_event_target(ctx, function->funcs+i, func_obj);
-        else
-            hres = jsdisp_propput_name(variable_obj, function->funcs[i].name, jsval_obj(func_obj));
+        hres = bind_event_target(ctx, function->funcs+i, func_obj);
         jsdisp_release(func_obj);
         if(FAILED(hres))
             return hres;
     }
 
-    for(i=0; i < function->var_cnt; i++) {
-        if(!(flags & EXEC_GLOBAL) || !lookup_global_members(ctx, function->variables[i], NULL)) {
-            DISPID id = 0;
+    if(flags & (EXEC_GLOBAL | EXEC_EVAL)) {
+        for(i=0; i < function->var_cnt; i++) {
+            TRACE("[%d] %s %d\n", i, debugstr_w(function->variables[i].name), function->variables[i].func_id);
+            if(function->variables[i].func_id != -1) {
+                jsdisp_t *func_obj;
 
-            hres = jsdisp_get_id(variable_obj, function->variables[i], fdexNameEnsure, &id);
-            if(FAILED(hres))
-                return hres;
+                hres = create_source_function(ctx, bytecode, function->funcs+function->variables[i].func_id, scope, &func_obj);
+                if(FAILED(hres))
+                    return hres;
+
+                hres = jsdisp_propput_name(variable_obj, function->variables[i].name, jsval_obj(func_obj));
+                jsdisp_release(func_obj);
+            }else if(!(flags & EXEC_GLOBAL) || !lookup_global_members(ctx, function->variables[i].name, NULL)) {
+                DISPID id = 0;
+
+                hres = jsdisp_get_id(variable_obj, function->variables[i].name, fdexNameEnsure, &id);
+                if(FAILED(hres))
+                    return hres;
+            }
         }
     }
 
@@ -2595,18 +2909,34 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
         }
     }
 
+    if(ctx->call_ctx && (flags & EXEC_EVAL)) {
+        hres = detach_variable_object(ctx, ctx->call_ctx, FALSE);
+        if(FAILED(hres))
+            return hres;
+    }
+
     frame = heap_alloc_zero(sizeof(*frame));
     if(!frame)
         return E_OUTOFMEMORY;
 
-    frame->bytecode = bytecode_addref(bytecode);
     frame->function = function;
+    frame->ret = jsval_undefined();
+    frame->argc = argc;
+    frame->bytecode = bytecode_addref(bytecode);
+
+    if(!(flags & (EXEC_GLOBAL|EXEC_EVAL))) {
+        hres = setup_scope(ctx, frame, scope, variable_obj, argc, argv);
+        if(FAILED(hres)) {
+            release_bytecode(frame->bytecode);
+            heap_free(frame);
+            return hres;
+        }
+    }else if(scope) {
+        frame->base_scope = frame->scope = scope_addref(scope);
+    }
+
     frame->ip = function->instr_off;
     frame->stack_base = ctx->stack_top;
-    frame->ret = jsval_undefined();
-    if(scope)
-        frame->base_scope = frame->scope = scope_addref(scope);
-
     if(this_obj)
         frame->this_obj = this_obj;
     else if(ctx->host_global)
@@ -2617,8 +2947,6 @@ HRESULT exec_source(script_ctx_t *ctx, DWORD flags, bytecode_t *bytecode, functi
 
     if(function_instance)
         frame->function_instance = jsdisp_addref(function_instance);
-    if(arguments_obj)
-        frame->arguments_obj = jsdisp_addref(arguments_obj);
 
     frame->flags = flags;
     frame->variable_obj = jsdisp_addref(variable_obj);
