@@ -182,7 +182,7 @@ static void ME_BeginRow(ME_WrapContext *wc)
   PARAFORMAT2 *pFmt;
   ME_DisplayItem *para = wc->pPara;
 
-  pFmt = para->member.para.pFmt;
+  pFmt = &para->member.para.fmt;
   wc->pRowStart = NULL;
   wc->bOverflown = FALSE;
   wc->pLastSplittableRun = NULL;
@@ -204,7 +204,7 @@ static void ME_BeginRow(ME_WrapContext *wc)
       width -= cell->prev_cell->member.cell.nRightBoundary;
     if (!cell->prev_cell)
     {
-      int rowIndent = ME_GetTableRowEnd(para)->member.para.pFmt->dxStartIndent;
+      int rowIndent = ME_GetTableRowEnd(para)->member.para.fmt.dxStartIndent;
       width -= rowIndent;
     }
     cell->nWidth = max(ME_twips2pointsX(wc->context, width), 0);
@@ -326,7 +326,7 @@ static void ME_InsertRowStart(ME_WrapContext *wc, const ME_DisplayItem *pEnd)
   para->nWidth = max(para->nWidth, width);
   row = ME_MakeRow(ascent+descent, ascent, width);
   if (wc->context->editor->bEmulateVersion10 && /* v1.0 - 3.0 */
-      (para->pFmt->dwMask & PFM_TABLE) && (para->pFmt->wEffects & PFE_TABLE))
+      (para->fmt.dwMask & PFM_TABLE) && (para->fmt.wEffects & PFE_TABLE))
   {
     /* The text was shifted down in ME_BeginRow so move the wrap context
      * back to where it should be. */
@@ -337,8 +337,8 @@ static void ME_InsertRowStart(ME_WrapContext *wc, const ME_DisplayItem *pEnd)
   row->member.row.pt = wc->pt;
   row->member.row.nLMargin = (!wc->nRow ? wc->nFirstMargin : wc->nLeftMargin);
   row->member.row.nRMargin = wc->nRightMargin;
-  assert(para->pFmt->dwMask & PFM_ALIGNMENT);
-  align = para->pFmt->wAlignment;
+  assert(para->fmt.dwMask & PFM_ALIGNMENT);
+  align = para->fmt.wAlignment;
   if (align == PFA_CENTER)
     shift = max((wc->nAvailWidth-width)/2, 0);
   if (align == PFA_RIGHT)
@@ -362,7 +362,7 @@ static void ME_InsertRowStart(ME_WrapContext *wc, const ME_DisplayItem *pEnd)
 static void ME_WrapEndParagraph(ME_WrapContext *wc, ME_DisplayItem *p)
 {
   ME_DisplayItem *para = wc->pPara;
-  PARAFORMAT2 *pFmt = para->member.para.pFmt;
+  PARAFORMAT2 *pFmt = &para->member.para.fmt;
   if (wc->pRowStart)
     ME_InsertRowStart(wc, p);
   if (wc->context->editor->bEmulateVersion10 && /* v1.0 - 3.0 */
@@ -685,19 +685,19 @@ static ME_DisplayItem *ME_WrapHandleRun(ME_WrapContext *wc, ME_DisplayItem *p)
 static int ME_GetParaLineSpace(ME_Context* c, ME_Paragraph* para)
 {
   int   sp = 0, ls = 0;
-  if (!(para->pFmt->dwMask & PFM_LINESPACING)) return 0;
+  if (!(para->fmt.dwMask & PFM_LINESPACING)) return 0;
 
   /* FIXME: how to compute simply the line space in ls ??? */
   /* FIXME: does line spacing include the line itself ??? */
-  switch (para->pFmt->bLineSpacingRule)
+  switch (para->fmt.bLineSpacingRule)
   {
   case 0:       sp = ls; break;
   case 1:       sp = (3 * ls) / 2; break;
   case 2:       sp = 2 * ls; break;
-  case 3:       sp = ME_twips2pointsY(c, para->pFmt->dyLineSpacing); if (sp < ls) sp = ls; break;
-  case 4:       sp = ME_twips2pointsY(c, para->pFmt->dyLineSpacing); break;
-  case 5:       sp = para->pFmt->dyLineSpacing / 20; break;
-  default: FIXME("Unsupported spacing rule value %d\n", para->pFmt->bLineSpacingRule);
+  case 3:       sp = ME_twips2pointsY(c, para->fmt.dyLineSpacing); if (sp < ls) sp = ls; break;
+  case 4:       sp = ME_twips2pointsY(c, para->fmt.dyLineSpacing); break;
+  case 5:       sp = para->fmt.dyLineSpacing / 20; break;
+  default: FIXME("Unsupported spacing rule value %d\n", para->fmt.bLineSpacingRule);
   }
   if (c->editor->nZoomNumerator == 0)
     return sp;
@@ -746,7 +746,7 @@ static HRESULT itemize_para( ME_Context *c, ME_DisplayItem *p )
 
     assert( p->type == diParagraph );
 
-    if (para->pFmt->dwMask & PFM_RTLPARA && para->pFmt->wEffects & PFE_RTLPARA)
+    if (para->fmt.dwMask & PFM_RTLPARA && para->fmt.wEffects & PFE_RTLPARA)
         state.uBidiLevel = 1;
 
     TRACE( "Base embedding level %d\n", state.uBidiLevel );
@@ -863,7 +863,7 @@ static void ME_WrapTextParagraph(ME_Context *c, ME_DisplayItem *tp) {
           shape_para( c, tp );
   }
 
-  pFmt = tp->member.para.pFmt;
+  pFmt = &tp->member.para.fmt;
 
   wc.context = c;
   wc.pPara = tp;
@@ -874,11 +874,16 @@ static void ME_WrapTextParagraph(ME_Context *c, ME_DisplayItem *tp) {
   } else {
     int dxStartIndent = pFmt->dxStartIndent;
     if (tp->member.para.pCell) {
-      dxStartIndent += ME_GetTableRowEnd(tp)->member.para.pFmt->dxOffset;
+      dxStartIndent += ME_GetTableRowEnd(tp)->member.para.fmt.dxOffset;
     }
     wc.nFirstMargin = ME_twips2pointsX(c, dxStartIndent);
     wc.nLeftMargin = wc.nFirstMargin + ME_twips2pointsX(c, pFmt->dxOffset);
     wc.nRightMargin = ME_twips2pointsX(c, pFmt->dxRightIndent);
+
+    if (wc.nFirstMargin < 0)
+        wc.nFirstMargin = 0;
+    if (wc.nLeftMargin < 0)
+        wc.nLeftMargin = 0;
   }
   if (c->editor->bEmulateVersion10 && /* v1.0 - 3.0 */
       pFmt->dwMask & PFM_TABLE && pFmt->wEffects & PFE_TABLE)
@@ -892,7 +897,7 @@ static void ME_WrapTextParagraph(ME_Context *c, ME_DisplayItem *tp) {
   if (!(pFmt->dwMask & PFM_TABLE && pFmt->wEffects & PFE_TABLE) &&
       pFmt->dwMask & PFM_BORDER)
   {
-    border = ME_GetParaBorderWidth(c, tp->member.para.pFmt->wBorders);
+    border = ME_GetParaBorderWidth(c, tp->member.para.fmt.wBorders);
     if (pFmt->wBorders & 1) {
       wc.nFirstMargin += border;
       wc.nLeftMargin += border;
@@ -919,7 +924,7 @@ static void ME_WrapTextParagraph(ME_Context *c, ME_DisplayItem *tp) {
   if (!(pFmt->dwMask & PFM_TABLE && pFmt->wEffects & PFE_TABLE) &&
       (pFmt->dwMask & PFM_BORDER) && (pFmt->wBorders & 8))
     wc.pt.y += border;
-  if (tp->member.para.pFmt->dwMask & PFM_SPACEAFTER)
+  if (tp->member.para.fmt.dwMask & PFM_SPACEAFTER)
     wc.pt.y += ME_twips2pointsY(c, pFmt->dySpaceAfter);
 
   tp->member.para.nFlags &= ~MEPF_REWRAP;
@@ -982,9 +987,9 @@ BOOL ME_WrapMarkedParagraphs(ME_TextEditor *editor)
         }
         c.pt.y += borderWidth;
       }
-      if (endRowPara->member.para.pFmt->dxStartIndent > 0)
+      if (endRowPara->member.para.fmt.dxStartIndent > 0)
       {
-        int dxStartIndent = endRowPara->member.para.pFmt->dxStartIndent;
+        int dxStartIndent = endRowPara->member.para.fmt.dxStartIndent;
         cell = ME_FindItemFwd(item, diCell);
         cell->member.cell.pt.x += ME_twips2pointsX(&c, dxStartIndent);
         c.pt.x = cell->member.cell.pt.x;
