@@ -942,6 +942,15 @@ VOID WINAPI Printers_UnregisterWindow(HANDLE hClassPidl, HWND hwnd)
     FIXME("(%p, %p) stub!\n", hClassPidl, hwnd);
 } 
 
+/*************************************************************************
+ * SHGetPropertyStoreForWindow [SHELL32.@]
+ */
+HRESULT WINAPI SHGetPropertyStoreForWindow(HWND hwnd, REFIID riid, void **ppv)
+{
+    FIXME("(%p %p %p) stub!\n", hwnd, riid, ppv);
+    return E_NOTIMPL;
+}
+
 /*************************************************************************
  * SHGetPropertyStoreFromParsingName [SHELL32.@]
  */
