@@ -425,6 +425,45 @@ static inline void write_bytes( struct writer *writer, const BYTE *bytes, ULONG
     writer->write_pos += len;
 }
 
+struct escape
+{
+    char        ch;
+    const char *entity;
+    ULONG       len;
+};
+static const struct escape escape_lt = { '<', "&lt;", 4 };
+static const struct escape escape_gt = { '>', "&gt;", 4 };
+static const struct escape escape_amp = { '&', "&amp;", 5 };
+static const struct escape escape_apos = { '\'', "&apos;", 6 };
+static const struct escape escape_quot = { '"', "&quot;", 6 };
+
+static HRESULT write_bytes_escape( struct writer *writer, const BYTE *bytes, ULONG len,
+                                   const struct escape **escapes, ULONG nb_escapes )
+{
+    ULONG i, j, size;
+    const BYTE *ptr;
+    HRESULT hr;
+
+    for (i = 0; i < len; i++)
+    {
+        ptr = &bytes[i];
+        size = 1;
+        for (j = 0; j < nb_escapes; j++)
+        {
+            if (bytes[i] == escapes[j]->ch)
+            {
+                ptr = (const BYTE *)escapes[j]->entity;
+                size = escapes[j]->len;
+                break;
+            }
+        }
+        if ((hr = write_grow_buffer( writer, size )) != S_OK) return hr;
+        write_bytes( writer, ptr, size );
+    }
+
+    return S_OK;
+}
+
 static HRESULT write_attribute( struct writer *writer, WS_XML_ATTRIBUTE *attr )
 {
     WS_XML_UTF8_TEXT *text = (WS_XML_UTF8_TEXT *)attr->value;
@@ -452,10 +491,17 @@ static HRESULT write_attribute( struct writer *writer, WS_XML_ATTRIBUTE *attr )
     write_bytes( writer, attr->localName->bytes, attr->localName->length );
     write_char( writer, '=' );
     write_char( writer, quote );
-    if (text) write_bytes( writer, text->value.bytes, text->value.length );
+    if (text)
+    {
+        const struct escape *escapes[3];
+        escapes[0] = attr->singleQuote ? &escape_apos : &escape_quot;
+        escapes[1] = &escape_lt;
+        escapes[2] = &escape_amp;
+        hr = write_bytes_escape( writer, text->value.bytes, text->value.length, escapes, 3 );
+    }
     write_char( writer, quote );
 
-    return S_OK;
+    return hr;
 }
 
 static inline BOOL is_current_namespace( struct writer *writer, const WS_XML_STRING *ns )
@@ -1285,9 +1331,20 @@ static HRESULT write_text( struct writer *writer )
     const WS_XML_UTF8_TEXT *utf8 = (const WS_XML_UTF8_TEXT *)text->text;
     HRESULT hr;
 
-    if ((hr = write_grow_buffer( writer, utf8->value.length )) != S_OK) return hr;
-    write_bytes( writer, utf8->value.bytes, utf8->value.length );
-    return S_OK;
+    if (!writer->current->parent) return WS_E_INVALID_FORMAT;
+    if (node_type( writer->current->parent ) == WS_XML_NODE_TYPE_ELEMENT)
+    {
+        const struct escape *escapes[3] = { &escape_lt, &escape_gt, &escape_amp };
+        return write_bytes_escape( writer, utf8->value.bytes, utf8->value.length, escapes, 3 );
+    }
+    else if (node_type( writer->current->parent ) == WS_XML_NODE_TYPE_CDATA)
+    {
+        if ((hr = write_grow_buffer( writer, utf8->value.length )) != S_OK) return hr;
+        write_bytes( writer, utf8->value.bytes, utf8->value.length );
+        return S_OK;
+    }
+
+    return WS_E_INVALID_FORMAT;
 }
 
 static HRESULT write_text_node( struct writer *writer, const WS_XML_TEXT *text )
@@ -1346,224 +1403,6 @@ static HRESULT write_type_text( struct writer *writer, WS_TYPE_MAPPING mapping,
     }
 }
 
-static HRESULT write_type_bool( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                const WS_BOOL_DESCRIPTION *desc, const BOOL *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[6]; /* "false" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_bool( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_int8( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                const WS_INT8_DESCRIPTION *desc, const INT8 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[5]; /* "-128" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_int8( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_int16( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                 const WS_INT16_DESCRIPTION *desc, const INT16 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[7]; /* "-32768" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_int16( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_int32( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                 const WS_INT32_DESCRIPTION *desc, const INT32 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[12]; /* "-2147483648" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_int32( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_int64( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                 const WS_INT64_DESCRIPTION *desc, const INT64 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[21]; /* "-9223372036854775808" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_int64( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_uint8( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                 const WS_UINT8_DESCRIPTION *desc, const UINT8 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[4]; /* "255" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_uint8( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_uint16( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                  const WS_UINT16_DESCRIPTION *desc, const UINT16 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[6]; /* "65535" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_uint16( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_uint32( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                  const WS_UINT32_DESCRIPTION *desc, const UINT32 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[11]; /* "4294967295" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_uint32( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_uint64( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                  const WS_UINT64_DESCRIPTION *desc, const UINT64 *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[21]; /* "18446744073709551615" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_uint64( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_guid( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                const WS_GUID_DESCRIPTION *desc, const GUID *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-    unsigned char buf[37]; /* "00000000-0000-0000-0000-000000000000" */
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = buf;
-    utf8.value.length  = format_guid( value, buf );
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
-static HRESULT write_type_string( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                  const WS_STRING_DESCRIPTION *desc, const WS_STRING *value )
-{
-    WS_XML_UTF16_TEXT utf16;
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf16.text.textType = WS_XML_TEXT_TYPE_UTF16;
-    utf16.bytes         = (BYTE *)value->chars;
-    utf16.byteCount     = value->length * sizeof(WCHAR);
-    return write_type_text( writer, mapping, &utf16.text );
-}
-
-static HRESULT write_type_wsz( struct writer *writer, WS_TYPE_MAPPING mapping,
-                               const WS_WSZ_DESCRIPTION *desc, const WCHAR *value )
-{
-    WS_XML_UTF16_TEXT utf16;
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf16.text.textType = WS_XML_TEXT_TYPE_UTF16;
-    utf16.bytes         = (BYTE *)value;
-    utf16.byteCount     = strlenW( value ) * sizeof(WCHAR);
-    return write_type_text( writer, mapping, &utf16.text );
-}
-
-static HRESULT write_type_xml_string( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                      const WS_XML_STRING_DESCRIPTION *desc, const WS_XML_STRING *value )
-{
-    WS_XML_UTF8_TEXT utf8;
-
-    if (desc)
-    {
-        FIXME( "description not supported\n" );
-        return E_NOTIMPL;
-    }
-    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
-    utf8.value.bytes   = value->bytes;
-    utf8.value.length  = value->length;
-    return write_type_text( writer, mapping, &utf8.text );
-}
-
 static HRESULT write_add_nil_attribute( struct writer *writer )
 {
     static const WS_XML_STRING prefix = {1, (BYTE *)"a"};
@@ -1577,6 +1416,358 @@ static HRESULT write_add_nil_attribute( struct writer *writer )
     return write_add_namespace_attribute( writer, &prefix, &ns, FALSE );
 }
 
+static HRESULT get_value_ptr( WS_WRITE_OPTION option, const void *value, ULONG size, ULONG expected_size,
+                              const void **ptr )
+{
+    switch (option)
+    {
+    case WS_WRITE_REQUIRED_VALUE:
+    case WS_WRITE_NILLABLE_VALUE:
+        if (!value || size != expected_size) return E_INVALIDARG;
+        *ptr = value;
+        return S_OK;
+
+    case WS_WRITE_REQUIRED_POINTER:
+        if (size != sizeof(const void *) || !(*ptr = *(const void **)value)) return E_INVALIDARG;
+        return S_OK;
+
+    case WS_WRITE_NILLABLE_POINTER:
+        if (size != sizeof(const void *)) return E_INVALIDARG;
+        *ptr = *(const void **)value;
+        return S_OK;
+
+    default:
+        return E_INVALIDARG;
+    }
+}
+
+static HRESULT write_type_bool( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                const WS_BOOL_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                const BOOL *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[6]; /* "false" */
+    const BOOL *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(BOOL), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_bool( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_int8( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                const WS_INT8_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                const BOOL *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[5]; /* "-128" */
+    const INT8 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(INT8), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_int8( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_int16( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                 const WS_INT16_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                 const BOOL *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[7]; /* "-32768" */
+    const INT16 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(INT16), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_int16( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_int32( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                 const WS_INT32_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                 const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[12]; /* "-2147483648" */
+    const INT32 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(INT32), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_int32( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_int64( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                 const WS_INT64_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                 const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[21]; /* "-9223372036854775808" */
+    const INT64 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(INT64), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_int64( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_uint8( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                 const WS_UINT8_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                 const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[4]; /* "255" */
+    const UINT8 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(UINT8), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_uint8( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_uint16( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                  const WS_UINT16_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                  const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[6]; /* "65535" */
+    const UINT16 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(UINT16), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_uint16( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_uint32( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                  const WS_UINT32_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                  const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[11]; /* "4294967295" */
+    const UINT32 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(UINT32), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_uint32( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_uint64( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                  const WS_UINT64_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                  const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[21]; /* "18446744073709551615" */
+    const UINT64 *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(UINT64), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_uint64( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_guid( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                const WS_GUID_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    unsigned char buf[37]; /* "00000000-0000-0000-0000-000000000000" */
+    const GUID *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(GUID), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = buf;
+    utf8.value.length  = format_guid( ptr, buf );
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
+static HRESULT write_type_string( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                  const WS_STRING_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                  const void *value, ULONG size )
+{
+    WS_XML_UTF16_TEXT utf16;
+    const WS_STRING *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(WS_STRING), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+    if (!ptr->length) return S_OK;
+
+    utf16.text.textType = WS_XML_TEXT_TYPE_UTF16;
+    utf16.bytes         = (BYTE *)ptr->chars;
+    utf16.byteCount     = ptr->length * sizeof(WCHAR);
+    return write_type_text( writer, mapping, &utf16.text );
+}
+
+static HRESULT write_type_wsz( struct writer *writer, WS_TYPE_MAPPING mapping,
+                               const WS_WSZ_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                               const void *value, ULONG size )
+{
+    WS_XML_UTF16_TEXT utf16;
+    const WCHAR *ptr;
+    HRESULT hr;
+    int len;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option || option == WS_WRITE_REQUIRED_VALUE || option == WS_WRITE_NILLABLE_VALUE) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, 0, (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+    if (!(len = strlenW( ptr ))) return S_OK;
+
+    utf16.text.textType = WS_XML_TEXT_TYPE_UTF16;
+    utf16.bytes         = (BYTE *)ptr;
+    utf16.byteCount     = len * sizeof(WCHAR);
+    return write_type_text( writer, mapping, &utf16.text );
+}
+
+static HRESULT write_type_xml_string( struct writer *writer, WS_TYPE_MAPPING mapping,
+                                      const WS_XML_STRING_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                      const void *value, ULONG size )
+{
+    WS_XML_UTF8_TEXT utf8;
+    const WS_XML_STRING *ptr;
+    HRESULT hr;
+
+    if (desc)
+    {
+        FIXME( "description not supported\n" );
+        return E_NOTIMPL;
+    }
+
+    if (!option) return E_INVALIDARG;
+    if ((hr = get_value_ptr( option, value, size, sizeof(WS_XML_STRING), (const void **)&ptr )) != S_OK) return hr;
+    if (option == WS_WRITE_NILLABLE_POINTER && !ptr) return write_add_nil_attribute( writer );
+    if (option == WS_WRITE_NILLABLE_VALUE && is_nil_value( value, size )) return write_add_nil_attribute( writer );
+    if (!ptr->length) return S_OK;
+
+    utf8.text.textType = WS_XML_TEXT_TYPE_UTF8;
+    utf8.value.bytes   = ptr->bytes;
+    utf8.value.length  = ptr->length;
+    return write_type_text( writer, mapping, &utf8.text );
+}
+
 static HRESULT write_type( struct writer *, WS_TYPE_MAPPING, WS_TYPE, const void *, WS_WRITE_OPTION,
                            const void *, ULONG );
 
@@ -1586,32 +1777,34 @@ static HRESULT write_type_struct_field( struct writer *writer, const WS_FIELD_DE
     HRESULT hr;
     WS_TYPE_MAPPING mapping;
     WS_WRITE_OPTION option;
+    ULONG field_options = desc->options;
 
-    if (desc->options & ~(WS_FIELD_POINTER|WS_FIELD_OPTIONAL|WS_FIELD_NILLABLE))
+    if (field_options & ~(WS_FIELD_POINTER|WS_FIELD_OPTIONAL|WS_FIELD_NILLABLE))
     {
         FIXME( "options 0x%x not supported\n", desc->options );
         return E_NOTIMPL;
     }
 
+    /*  zero-terminated strings are always pointers */
+    if (desc->type == WS_WSZ_TYPE) field_options |= WS_FIELD_POINTER;
+
     if (is_nil_value( value, size ))
     {
-        if (desc->options & WS_FIELD_NILLABLE)
+        if (field_options & WS_FIELD_NILLABLE)
         {
-            if (desc->options & WS_FIELD_POINTER)
-                option = WS_WRITE_NILLABLE_POINTER;
-            else
-                option = WS_WRITE_NILLABLE_VALUE;
+            if (field_options & WS_FIELD_POINTER) option = WS_WRITE_NILLABLE_POINTER;
+            else option = WS_WRITE_NILLABLE_VALUE;
         }
         else
         {
-            if (desc->options & WS_FIELD_OPTIONAL) return S_OK;
+            if (field_options & WS_FIELD_OPTIONAL) return S_OK;
             return E_INVALIDARG;
         }
     }
     else
     {
-        if (desc->options & WS_FIELD_POINTER) option = WS_WRITE_REQUIRED_POINTER;
-        else option = 0;
+        if (field_options & WS_FIELD_POINTER) option = WS_WRITE_REQUIRED_POINTER;
+        else option = WS_WRITE_REQUIRED_VALUE;
     }
 
     switch (desc->mapping)
@@ -1627,10 +1820,6 @@ static HRESULT write_type_struct_field( struct writer *writer, const WS_FIELD_DE
 
     case WS_ELEMENT_FIELD_MAPPING:
         if ((hr = write_element_node( writer, NULL, desc->localName, desc->ns )) != S_OK) return hr;
-        if (option == WS_WRITE_NILLABLE_VALUE || option == WS_WRITE_NILLABLE_POINTER)
-        {
-            if ((hr = write_add_nil_attribute( writer )) != S_OK) return hr;
-        }
         mapping = WS_ELEMENT_TYPE_MAPPING;
         break;
 
@@ -1656,11 +1845,8 @@ static HRESULT write_type_struct_field( struct writer *writer, const WS_FIELD_DE
         return E_NOTIMPL;
     }
 
-    if (option != WS_WRITE_NILLABLE_VALUE && option != WS_WRITE_NILLABLE_POINTER)
-    {
-        if ((hr = write_type( writer, mapping, desc->type, desc->typeDescription, option, value, size )) != S_OK)
-            return hr;
-    }
+    if ((hr = write_type( writer, mapping, desc->type, desc->typeDescription, option, value, size )) != S_OK)
+        return hr;
 
     switch (mapping)
     {
@@ -1685,156 +1871,78 @@ static ULONG get_field_size( const WS_STRUCT_DESCRIPTION *desc, ULONG index )
 }
 
 static HRESULT write_type_struct( struct writer *writer, WS_TYPE_MAPPING mapping,
-                                  const WS_STRUCT_DESCRIPTION *desc, const void *value )
+                                  const WS_STRUCT_DESCRIPTION *desc, WS_WRITE_OPTION option,
+                                  const void *value, ULONG size )
 {
-    ULONG i, size;
+    ULONG i, field_size;
+    const void *ptr, *field_ptr;
     HRESULT hr;
-    const char *ptr;
 
+    if (!desc) return E_INVALIDARG;
     if (desc->structOptions) FIXME( "struct options 0x%x not supported\n", desc->structOptions );
 
+    if ((hr = get_value_ptr( option, value, size, desc->size, (const void **)&ptr )) != S_OK) return hr;
+
     for (i = 0; i < desc->fieldCount; i++)
     {
-        ptr = (const char *)value + desc->fields[i]->offset;
-        size = get_field_size( desc, i );
-        if ((hr = write_type_struct_field( writer, desc->fields[i], ptr, size )) != S_OK)
+        field_ptr = (const char *)ptr + desc->fields[i]->offset;
+        field_size = get_field_size( desc, i );
+        if ((hr = write_type_struct_field( writer, desc->fields[i], field_ptr, field_size )) != S_OK)
             return hr;
     }
 
     return S_OK;
 }
 
-static HRESULT get_value_ptr( WS_WRITE_OPTION option, const void *value, ULONG size, const void **ptr )
-{
-    switch (option)
-    {
-    case WS_WRITE_REQUIRED_VALUE:
-    case WS_WRITE_NILLABLE_VALUE:
-        if (!value || !size) return E_INVALIDARG;
-        *ptr = value;
-        return S_OK;
-
-    case WS_WRITE_REQUIRED_POINTER:
-    case WS_WRITE_NILLABLE_POINTER:
-        if (size != sizeof(const void *) || !(*ptr = *(const void **)value)) return E_INVALIDARG;
-        return S_OK;
-
-    default:
-        FIXME( "option %08x not supported\n", option );
-        return E_NOTIMPL;
-    }
-}
 
 static HRESULT write_type( struct writer *writer, WS_TYPE_MAPPING mapping, WS_TYPE type,
                            const void *desc, WS_WRITE_OPTION option, const void *value,
                            ULONG size )
 {
-    HRESULT hr;
-
     switch (type)
     {
     case WS_STRUCT_TYPE:
-    {
-        const void *ptr;
-        if (!desc) return E_INVALIDARG;
-        if (!option) option = WS_WRITE_REQUIRED_POINTER;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_struct( writer, mapping, desc, ptr );
-    }
+        return write_type_struct( writer, mapping, desc, option, value, size );
+
     case WS_BOOL_TYPE:
-    {
-        const BOOL *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_bool( writer, mapping, desc, ptr );
-    }
+        return write_type_bool( writer, mapping, desc, option, value, size );
+
     case WS_INT8_TYPE:
-    {
-        const INT8 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_int8( writer, mapping, desc, ptr );
-    }
+        return write_type_int8( writer, mapping, desc, option, value, size );
+
     case WS_INT16_TYPE:
-    {
-        const INT16 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_int16( writer, mapping, desc, ptr );
-    }
+        return write_type_int16( writer, mapping, desc, option, value, size );
+
     case WS_INT32_TYPE:
-    {
-        const INT32 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_int32( writer, mapping, desc, ptr );
-    }
+        return write_type_int32( writer, mapping, desc, option, value, size );
+
     case WS_INT64_TYPE:
-    {
-        const INT64 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_int64( writer, mapping, desc, ptr );
-    }
+        return write_type_int64( writer, mapping, desc, option, value, size );
+
     case WS_UINT8_TYPE:
-    {
-        const UINT8 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_uint8( writer, mapping, desc, ptr );
-    }
+        return write_type_uint8( writer, mapping, desc, option, value, size );
+
     case WS_UINT16_TYPE:
-    {
-        const UINT16 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_uint16( writer, mapping, desc, ptr );
-    }
+        return write_type_uint16( writer, mapping, desc, option, value, size );
+
     case WS_UINT32_TYPE:
-    {
-        const UINT32 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_uint32( writer, mapping, desc, ptr );
-    }
+        return write_type_uint32( writer, mapping, desc, option, value, size );
+
     case WS_UINT64_TYPE:
-    {
-        const UINT64 *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_uint64( writer, mapping, desc, ptr );
-    }
+        return write_type_uint64( writer, mapping, desc, option, value, size );
+
     case WS_GUID_TYPE:
-    {
-        const GUID *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_guid( writer, mapping, desc, ptr );
-    }
+        return write_type_guid( writer, mapping, desc, option, value, size );
+
     case WS_STRING_TYPE:
-    {
-        const WS_STRING *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_string( writer, mapping, desc, ptr );
-    }
+        return write_type_string( writer, mapping, desc, option, value, size );
+
     case WS_WSZ_TYPE:
-    {
-        const WCHAR *ptr;
+        return write_type_wsz( writer, mapping, desc, option, value, size );
 
-        if (option == WS_WRITE_REQUIRED_VALUE) return E_INVALIDARG;
-
-        if (!option) option = WS_WRITE_REQUIRED_POINTER;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_wsz( writer, mapping, desc, ptr );
-    }
     case WS_XML_STRING_TYPE:
-    {
-        const WS_XML_STRING *ptr;
-        if (!option) option = WS_WRITE_REQUIRED_VALUE;
-        if ((hr = get_value_ptr( option, value, size, (const void **)&ptr )) != S_OK) return hr;
-        return write_type_xml_string( writer, mapping, desc, ptr );
-    }
+        return write_type_xml_string( writer, mapping, desc, option, value, size );
+
     default:
         FIXME( "type %u not supported\n", type );
         return E_NOTIMPL;
