@@ -1083,7 +1083,7 @@ static void state_init_default(struct wined3d_state *state, const struct wined3d
     state->render_states[WINED3D_RS_LASTPIXEL] = TRUE;
     state->render_states[WINED3D_RS_SRCBLEND] = WINED3D_BLEND_ONE;
     state->render_states[WINED3D_RS_DESTBLEND] = WINED3D_BLEND_ZERO;
-    state->render_states[WINED3D_RS_CULLMODE] = WINED3D_CULL_CCW;
+    state->render_states[WINED3D_RS_CULLMODE] = WINED3D_CULL_BACK;
     state->render_states[WINED3D_RS_ZFUNC] = WINED3D_CMP_LESSEQUAL;
     state->render_states[WINED3D_RS_ALPHAFUNC] = WINED3D_CMP_ALWAYS;
     state->render_states[WINED3D_RS_ALPHAREF] = 0;
