@@ -1596,6 +1596,7 @@ static void test_fragmentsize(void)
     rc = waveOutClose(wout);
     ok(rc == MMSYSERR_NOERROR, "waveOutClose failed: %s\n", wave_out_error(rc));
 
+    HeapFree(GetProcessHeap(), 0, hdr[0].lpData);
     CloseHandle(hevent);
 }
 
@@ -1621,10 +1622,12 @@ static void create_wav_file(char *temp_file)
     ok(h != NULL, "Can't open temp_file\n");
 
     riff_chunk.fccType = mmioFOURCC('W','A','V','E');
+    riff_chunk.cksize = 0;
     rc = mmioCreateChunk(h, &riff_chunk, MMIO_CREATERIFF);
     ok(rc == MMSYSERR_NOERROR, "mmioCreateChunk failed, got %u\n", rc);
 
     chunk.ckid = mmioFOURCC('f','m','t',' ');
+    chunk.cksize = 0;
     rc = mmioCreateChunk(h, &chunk, 0);
     ok(rc == MMSYSERR_NOERROR, "mmioCreateChunk failed, got %u\n", rc);
     written = mmioWrite(h, (char*)&format, sizeof(format));
