@@ -78,6 +78,8 @@ static const struct object_ops irp_call_ops =
     default_get_sd,                   /* get_sd */
     default_set_sd,                   /* set_sd */
     no_lookup_name,                   /* lookup_name */
+    no_link_name,                     /* link_name */
+    NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
     no_close_handle,                  /* close_handle */
     irp_call_destroy                  /* destroy */
@@ -112,6 +114,8 @@ static const struct object_ops device_manager_ops =
     default_get_sd,                   /* get_sd */
     default_set_sd,                   /* set_sd */
     no_lookup_name,                   /* lookup_name */
+    no_link_name,                     /* link_name */
+    NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
     no_close_handle,                  /* close_handle */
     device_manager_destroy            /* destroy */
@@ -151,6 +155,8 @@ static const struct object_ops device_ops =
     default_get_sd,                   /* get_sd */
     default_set_sd,                   /* set_sd */
     no_lookup_name,                   /* lookup_name */
+    directory_link_name,              /* link_name */
+    default_unlink_name,              /* unlink_name */
     device_open_file,                 /* open_file */
     no_close_handle,                  /* close_handle */
     device_destroy                    /* destroy */
@@ -197,6 +203,8 @@ static const struct object_ops device_file_ops =
     default_get_sd,                   /* get_sd */
     default_set_sd,                   /* set_sd */
     no_lookup_name,                   /* lookup_name */
+    no_link_name,                     /* link_name */
+    NULL,                             /* unlink_name */
     no_open_file,                     /* open_file */
     device_file_close_handle,         /* close_handle */
     device_file_destroy               /* destroy */
@@ -314,11 +322,7 @@ static void set_irp_result( struct irp_call *irp, unsigned int status,
 
 static void device_dump( struct object *obj, int verbose )
 {
-    struct device *device = (struct device *)obj;
-
-    fprintf( stderr, "Device " );
-    dump_object_name( &device->obj );
-    fputc( '\n', stderr );
+    fputs( "Device\n", stderr );
 }
 
 static struct object_type *device_get_type( struct object *obj )
@@ -595,12 +599,12 @@ static obj_handle_t device_file_ioctl( struct fd *fd, ioctl_code_t code, const a
     return handle;
 }
 
-static struct device *create_device( struct directory *root, const struct unicode_str *name,
+static struct device *create_device( struct object *root, const struct unicode_str *name,
                                      struct device_manager *manager, unsigned int attr )
 {
     struct device *device;
 
-    if ((device = create_named_object_dir( root, name, attr, &device_ops )))
+    if ((device = create_named_object( root, &device_ops, name, attr, NULL )))
     {
         if (get_error() != STATUS_OBJECT_NAME_EXISTS)
         {
@@ -614,19 +618,18 @@ static struct device *create_device( struct directory *root, const struct unicod
     return device;
 }
 
-struct device *create_unix_device( struct directory *root, const struct unicode_str *name,
+struct object *create_unix_device( struct object *root, const struct unicode_str *name,
                                    const char *unix_path )
 {
     struct device *device;
 
-    if ((device = create_named_object_dir( root, name, 0, &device_ops )))
+    if ((device = create_named_object( root, &device_ops, name, 0, NULL )))
     {
         device->unix_path = strdup( unix_path );
         device->manager = NULL;  /* no manager, requests go straight to the Unix device */
         list_init( &device->files );
-        make_object_static( &device->obj );
     }
-    return device;
+    return &device->obj;
 
 }
 
@@ -712,16 +715,15 @@ DECL_HANDLER(create_device_manager)
 DECL_HANDLER(create_device)
 {
     struct device *device;
-    struct unicode_str name;
+    struct unicode_str name = get_req_unicode_str();
     struct device_manager *manager;
-    struct directory *root = NULL;
+    struct object *root = NULL;
 
     if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
                                                              0, &device_manager_ops )))
         return;
 
-    get_req_unicode_str( &name );
-    if (req->rootdir && !(root = get_directory_obj( current->process, req->rootdir, 0 )))
+    if (req->rootdir && !(root = get_directory_obj( current->process, req->rootdir )))
     {
         release_object( manager );
         return;
@@ -807,11 +809,6 @@ DECL_HANDLER(get_next_device_request)
 DECL_HANDLER(set_irp_result)
 {
     struct irp_call *irp;
-    struct device_manager *manager;
-
-    if (!(manager = (struct device_manager *)get_handle_obj( current->process, req->manager,
-                                                             0, &device_manager_ops )))
-        return;
 
     if ((irp = (struct irp_call *)get_handle_obj( current->process, req->handle, 0, &irp_call_ops )))
     {
@@ -820,7 +817,6 @@ DECL_HANDLER(set_irp_result)
         close_handle( current->process, req->handle );  /* avoid an extra round-trip for close */
         release_object( irp );
     }
-    release_object( manager );
 }
 
 
